/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "childwndmfile.h"
#include "wdgmfileeditor.h"
#include "ui_wdgmfileeditor.h"
#include <QMenu>
#include <QMenuBar>
#include <QFile>
#include <QTextStream>
#include <QFileInfo>
#include <QFileDialog>

#include "mainwnd.h"
#include "ui_mainwnd.h"

#include <QMessageBox>

treeItemMFile::treeItemMFile(guiProject * p, childWndMFile * dlg, QTreeWidgetItem *parent):
    abstractPrjTreeItem(parent)
{
    this->setText(0, dlg->windowTitle());
    this->setToolTip(0, dlg->documentName);
    this->setIcon(0, QPixmap(":/img/mFileItem"));
    this->dlg = dlg;
    this->associatedPrj = p;
}

void treeItemMFile::activate()
{
    int tabNro = -1;
    bool finded = false;
    int k;
    for(k = 0; (k < this->associatedPrj->getProjectWidget()->ui->docTabs->count()) && !finded; k++){
        if(this->associatedPrj->getProjectWidget()->ui->docTabs->tabText(k) == dlg->getName()){
            finded = true;
            this->associatedPrj->getProjectWidget()->ui->docTabs->setCurrentIndex(k);
        }
    }
    if(!finded){
        tabNro = this->associatedPrj->getProjectWidget()->ui->docTabs->addTab(dlg, dlg->getName());
        this->associatedPrj->getProjectWidget()->ui->docTabs->setCurrentIndex(tabNro);
    }
}

void treeItemMFile::clicked()
{
    this->activate();
}

void treeItemMFile::populateMenu(QMenu &m)
{
    m.addAction(tr("Delete") + " " + dlg->windowTitle(), this, SLOT(actDelete(bool)));
}

void treeItemMFile::actDelete(bool checked __attribute__((unused)))
{
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(mainWnd::getMainWnd(),
                                  QObject::tr("Warning!!! This operation cannot be undone"),
                                  QObject::tr("You are trying to delete the file %1, this operation can't be undone. Are you sure to delete this file?").arg(this->dlg->documentName),
                                  QMessageBox::Yes | QMessageBox::No);
    if(reply == QMessageBox::Yes)
    {
        QFile toDelete(this->dlg->documentName);
        toDelete.remove();
        QFileInfo fi(this->dlg->documentName);

        QFile runCopy(this->associatedPrj->runPath() + QDir::separator() + "fcn" + QDir::separator() + fi.fileName());
        if(runCopy.exists())
            runCopy.remove();

        QFile exportCopy(this->associatedPrj->exportPath() + QDir::separator() + "fcn" + QDir::separator() + fi.fileName());
        if(exportCopy.exists())
            exportCopy.remove();

        this->dlg->close();
        this->associatedPrj->items.removeAll(this->dlg);
        bool finded = false;
        for(int k = 0; (k < this->associatedPrj->getProjectWidget()->ui->docTabs->count()) && (!finded); k++){
            if(this->associatedPrj->getProjectWidget()->ui->docTabs->tabText(k) == this->dlg->documentName){
                this->associatedPrj->getProjectWidget()->ui->docTabs->widget(k);
                this->associatedPrj->getProjectWidget()->ui->docTabs->removeTab(k);
                finded = true;
            }
        }
        this->dlg->deleteLater();
        this->associatedPrj->getProjectWidget()->updateTreeNames();
    }
}

void treeItemMFile::selectProject()
{
    //mainWnd::getPrjWnd()->setActiveProject(this->associatedPrj);
    QMessageBox::information(0, "Cuidado", "Select project");

}


childWndMFile::childWndMFile(QWidget *parent, guiProject * prj)
    :QWidget(parent),projectItem("MFile", prj)
{
    srcEditorWidget = new wdgMFileEditor(this);
    this->vGroupName = tr("M files");
    //this->setWidget(srcEditorWidget);
    //this->setGeometry(1, 1, 480, 320);
    QHBoxLayout *bl = new QHBoxLayout();
    this->setLayout(bl);
    bl->addWidget(srcEditorWidget);
    bl->setMargin(0);
    bl->setSpacing(0);
    this->setWindowIcon(QPixmap(":/img/img/icono.svg"));
}

wdgMFileEditor * childWndMFile::getSrcEditor()
{
    return this->srcEditorWidget;
}

void childWndMFile::save()
{    
    if(!this->documentHaveName)
    {
      QString fileName = QFileDialog::getSaveFileName(this, tr("Select destination file name"), QApplication::applicationDirPath(), tr("Octave script (*.m);;All files (*.*);;"), 0);
      if(!fileName.isNull())
      {
          this->documentHaveName = true;
          this->documentName = fileName;
          this->prj->disableFSWatch();
          QFile f(fileName);
          QTextStream sTxt(&f);
          f.open(QIODevice::WriteOnly | QIODevice::Text);          
          sTxt << this->srcEditorWidget->ui->srcEditor->text();
          f.close();
          this->setWindowModified(false);
          QFileInfo fileInfo(fileName);
          this->setWindowTitle(fileInfo.fileName());
          this->setLastModified(fileInfo.lastModified());
          this->prj->enableFSWatch();
      }
    }
    else
    {
        this->prj->disableFSWatch();
        QFile f(this->documentName);
        f.open(QIODevice::WriteOnly | QIODevice::Text);
        QTextStream sTxt(&f);
        sTxt << this->srcEditorWidget->ui->srcEditor->text();
        f.close();
        QFileInfo fileInfo(this->documentName);
        this->setLastModified(fileInfo.lastModified());
        this->setWindowModified(false);
        this->prj->enableFSWatch();
    }
}

void childWndMFile::save(QString fileName)
{
    QFile f(fileName);
    this->documentHaveName = true;
    this->documentName = fileName;
    this->setWindowModified(false);
    this->prj->disableFSWatch();
    f.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream sTxt(&f);
    sTxt << this->srcEditorWidget->ui->srcEditor->text();
    f.close();
    QFileInfo fileInfo(fileName);
    this->setWindowTitle(fileInfo.fileName());
    this->setLastModified(fileInfo.lastModified());
    this->prj->enableFSWatch();
}

void childWndMFile::saveAs()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Select destination file name"), QApplication::applicationDirPath(), tr("Octave script (*.m);;All files (*);;"), 0);
    if(!fileName.isNull())
    {
        this->documentHaveName = true;
        this->documentName = fileName;
        QFile f(fileName);
        f.open(QIODevice::WriteOnly | QIODevice::Text);
        QTextStream sTxt(&f);
        sTxt << this->srcEditorWidget->ui->srcEditor->text();
        f.close();
        this->setWindowModified(false);
        QFileInfo fileInfo(fileName);
        this->setWindowTitle(fileInfo.fileName());
     }
}

void childWndMFile::load(QString fn)
{
    QFile f(fn);
    QFileInfo fileInfo(fn);
    f.open(QIODevice::ReadOnly | QIODevice::Text);
    this->srcEditorWidget->ui->srcEditor->clear();
    this->srcEditorWidget->ui->srcEditor->setText(f.readAll());
    f.close();
    this->documentHaveName = true;
    this->documentName = fn;
    this->srcEditorWidget->setWindowModified(false);
    this->srcEditorWidget->ui->srcEditor->setModified(false);
    this->setWindowTitle(this->getName());
    this->setLastModified(fileInfo.lastModified());
}

void childWndMFile::run(QString path __attribute__((unused)))
{

}

void childWndMFile::exportAsScript(QString aPath)
{
    QDir fcnPath(aPath);
    if(!fcnPath.exists("fcn"))
        fcnPath.mkdir("fcn");
    QFile f(aPath + "/fcn/" + getName());
    f.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream sTxt(&f);
    sTxt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());
    sTxt << this->srcEditorWidget->ui->srcEditor->text();
    f.close();
}

void childWndMFile::activateAtLine(int line, int col)
{
    int tabNro = -1;
    bool finded = false;
    int k;
    for(k = 0; (k < this->prj->getProjectWidget()->ui->docTabs->count()) && !finded; k++){
        if(this->prj->getProjectWidget()->ui->docTabs->tabText(k) == this->getName()){
            finded = true;
            this->prj->getProjectWidget()->ui->docTabs->setCurrentIndex(k);
        }
    }
    if(!finded){
        tabNro = this->prj->getProjectWidget()->ui->docTabs->addTab(this, this->getName());
        this->prj->getProjectWidget()->ui->docTabs->setCurrentIndex(tabNro);
    }
    if((line != -1) && (col != -1)){
        this->srcEditorWidget->ui->srcEditor->setCursorPosition(line -1, col - 1);
        this->srcEditorWidget->ui->srcEditor->setFocus();
    }
}

void childWndMFile::closingProject()
{
    this->close();
    mainWnd::getMainWnd()->ui->mdiArea->removeSubWindow(this);
}

QString childWndMFile::getName()
{
    QFileInfo fileInfo(documentName);
    return fileInfo.fileName();
}

bool childWndMFile::isModified()
{
    return  isWindowModified() | srcEditorWidget->ui->srcEditor->isModified();
}


abstractPrjTreeItem * childWndMFile::treeItemWidget(abstractPrjTreeItem *treeItem)
{
    return new treeItemMFile(dynamic_cast<treeItemGroup*>(treeItem)->getPrj(), this, treeItem);
}
