/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#ifndef CONFIGSETTINGS_H
#define CONFIGSETTINGS_H
#include <QSettings>
#include <QFont>
#include "dlgappconfig.h"
typedef enum {
    cvDontCheck = 0,
    cvCheck = 1,
    cvAsk = 2
}stCheckVersion ;

class configSettings
{
private:
    QString octavePath;
    QString workingDir;
    QString tempDir;
    QStringList libPaths;
    QString language;
    QString qt5PlugginPath;
    QString gzipPath;
    QString tarPath;
    QFont ctrlFont;
    QFont editorFont;
    int editorTabSize;
    QStringList recentPrj;
    bool forceCodec;
    QString outputCodec;
    stCheckVersion checkNewVersion;
                        // 0 -> Don't check
                         // 1 -> Check
                         // 2 -> Its not known (Ask)
    unsigned int lastVersionCheck;
public:
    configSettings();
    void loadCfg();
    void saveCfg();
    bool octavePathOk();
    bool tmpDirOk();

    bool gzipPathOk();
    bool tarPathOk();

    void paramToWnd(dlgAppConfig *wnd);
    void wndToParam(dlgAppConfig *wnd);

    int getEditorTabSize();

    QString getQt5PlugginPath();
    void setQt5PlugginPath(QString newPath);

    QString getOctavePath();
    void setOctavePath(QString newPath);

    QString getTempDir();

    QString getLanguage();
    void setLanguage(QString newLan);

    QString getWorkingDir(void);
    void setWorkingDir(QString wd);

    QString getGzipPath();
    void setGzipPath(QString newPath);

    QString getTarPath();
    void setTarPath(QString newPath);

    QFont getCtrlFont();
    void setCtrlFont(QFont newFont);

    QFont getEditorFont();
    void setEditorFont(QFont newFont);

    QStringList getLibPaths();

    void registerRecentPrj(QString prjPath);
    QString getRecentPrj(int prjNumber);
    int getRecentPrjCount(void);

    bool getForceOutputCodec(void);
    void setForceOutputCodec(bool force);

    QString getOutputCodec(void);
    void setOutputCodec(QString newCodec);

    stCheckVersion getCheckVersion(void);
    void setCheckVersion(stCheckVersion val);

    unsigned int getLastVersionCheck(void);
    void setLastVersionCheck(unsigned int);
    ~configSettings();
};

#endif // CONFIGSETTINGS_H
