#include "dlgtbleditor.h"
#include "ui_dlgtbleditor.h"
#include <QDebug>
#include "tablectrl.h"

dlgTblEditor::dlgTblEditor(QString defCfg, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::dlgTblEditor)
{
    ui->setupUi(this);
    setValue(defCfg);
}

QString dlgTblEditor::getValue(void){
  QString ret;
  int k;
  int f, c;
  ret  = "colCount = " + QString::number(ui->tblValues->columnCount()) + " | ";
  ret += "rowCount = " + QString::number(ui->tblValues->rowCount()) + " | ";
  ret += "defineWith = " + ui->cmbDef->currentText() + " | ";
  if(ui->cbUseRowNum->isChecked()){
   ret += "rowNames = numbered | ";
  }
  else{
      ret += "rowNames = ";
      for(k = 0; k < ui->tblVTitles->rowCount(); k++){
          ret += ui->tblVTitles->item(k, 0)->text();
          if(k != ui->tblVTitles->rowCount() - 1){
              ret += ",";
          }
      }
      ret += " | ";
  }
  if(ui->cbUseColNum->isChecked())
   ret += "columnNames = numbered | ";
  else{
      ret += "columnNames = ";
      for(k = 0; k < ui->tblHTitles->columnCount(); k++){
          ret += ui->tblHTitles->item(0, k)->text();
          if(k != ui->tblHTitles->columnCount() - 1){
              ret += ",";
          }
      }
      ret += " | ";
  }
  ret += "columnWidth = ";
  for(k = 0; k < ui->tblColWidth->columnCount(); k++){
      ret += ((QComboBox *)ui->tblColWidth->cellWidget(0, k))->currentText();
      if(k < ui->tblColWidth->columnCount() - 1){
          ret += ",";
      }
  }
  ret += " | ";

  ret += "columnFormat = ";
  for(k = 0; k < ui->tblColFormat->columnCount(); k++){
      ret += ((QComboBox *)ui->tblColFormat->cellWidget(0, k))->currentText();
      if(k < ui->tblColFormat->columnCount() - 1){
          ret += ",";
      }
  }
  ret += " | ";
  ret += "columneditable = ";
  for(k = 0; k < ui->tblColEdit->columnCount(); k++){
      ret += ((QComboBox *)ui->tblColEdit->cellWidget(0, k))->currentText();
      if(k < ui->tblColEdit->columnCount() - 1){
          ret += ",";
      }
  }
  ret += " | ";
  ret += "data = ";
  for(f = 0; f < ui->tblValues->rowCount(); f++){
      for(c = 0; c < ui->tblValues->columnCount(); c++){
          if(ui->tblValues->item(f, c)){
            ret += ui->tblValues->item(f, c)->text();
          }
          if(c < ui->tblValues->columnCount() - 1)
              ret += ",";
      }
      if(f < ui->tblValues->rowCount() - 1)
          ret += ";";
  }
  return ret;
}

void dlgTblEditor::setValue(QString newVal){
    int f, c, k;
    QComboBox *cmbColWidth;
    QComboBox *cmbColFmt;
    QTableWidgetItem *cell;
    tblDataConfig tblCfg(newVal);
    ui->edColCount->setValue(tblCfg.getColCount());
    ui->edRowCount->setValue(tblCfg.getRowCount());
    ui->tblColFormat->setColumnCount(tblCfg.getColCount());
    ui->tblColEdit->setColumnCount(tblCfg.getColCount());
    ui->tblColWidth->setColumnCount(tblCfg.getColCount());
    ui->tblHTitles->setColumnCount(tblCfg.getColCount());
    ui->tblVTitles->setRowCount(tblCfg.getRowCount());
    ui->tblValues->setColumnCount(tblCfg.getColCount());
    ui->tblValues->setRowCount(tblCfg.getRowCount());
    ui->cmbDef->setCurrentText(tblCfg.getBaseDef());
    ui->cbUseColNum->setChecked(tblCfg.getNumberedCols());
    ui->cbUseRowNum->setChecked(tblCfg.getNumberedRows());
    for(k = 0; k < tblCfg.getColCount(); k++){
        cell = new QTableWidgetItem(tblCfg.getColTitle(k));
        cell->setTextAlignment(Qt::AlignCenter);
        if(ui->cbUseColNum->isChecked()){
            cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
        else{
            cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
        }
        ui->tblHTitles->setItem(0, k, cell);
    }

    for(k = 0; k < tblCfg.getRowCount(); k++){
        cell = new QTableWidgetItem(tblCfg.getRowTitle(k));
        cell->setTextAlignment(Qt::AlignCenter);
        if(ui->cbUseRowNum->isChecked()){
            cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
        else{
            cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
        }
        ui->tblVTitles->setItem(k, 0, cell);
    }

    for(k = 0; k < tblCfg.getColCount(); k++){
        cmbColWidth = new QComboBox(ui->tblColWidth);
        cmbColWidth->addItem("auto");
        cmbColWidth->addItem("fit");
        cmbColWidth->addItem("1x");
        cmbColWidth->addItem("2x");
        cmbColWidth->addItem("25");
        cmbColWidth->addItem("50");
        cmbColWidth->setToolTip("Column " + QString::number(k + 1));
        cmbColWidth->setEditable(true);
        cmbColWidth->setCurrentText(tblCfg.getColWidth(k));
        ui->tblColWidth->setCellWidget(0, k, cmbColWidth);
    }

    for(k = 0; k < tblCfg.getColCount(); k++){
        cmbColFmt = new QComboBox(ui->tblColWidth);
        cmbColFmt->addItem("char");
        cmbColFmt->addItem("logical");
        cmbColFmt->addItem("numeric");
        cmbColFmt->setEditable(true);
        cmbColFmt->setToolTip("Column " + QString::number(k + 1));
        cmbColFmt->setCurrentText(tblCfg.getColFmt(k));
        ui->tblColFormat->setCellWidget(0, k, cmbColFmt);
    }

    for(c = 0; c < tblCfg.getColCount(); c++){
        for(f = 0; f < tblCfg.getRowCount(); f++){
            cell = new QTableWidgetItem(tblCfg.getData(f, c));
            cell->setTextAlignment(Qt::AlignCenter);
            ui->tblValues->setItem(f, c, cell);
        }
    }
    for(k = 0; k < tblCfg.getColCount(); k++){
        cmbColWidth = new QComboBox(ui->tblColWidth);
        cmbColWidth->addItem("true");
        cmbColWidth->addItem("false");
        cmbColWidth->setToolTip("Column " + QString::number(k + 1));
        cmbColWidth->setEditable(false);
        cmbColWidth->setCurrentText(tblCfg.getColEdit(k));
        ui->tblColEdit->setCellWidget(0, k, cmbColWidth);
    }
}

dlgTblEditor::~dlgTblEditor()
{
    delete ui;
}


void dlgTblEditor::on_edRowCount_valueChanged(int arg1)
{
    QTableWidgetItem * cell;
    int k;
    int prevRowCount = ui->tblValues->rowCount();
    ui->tblValues->setRowCount(arg1);
    ui->tblVTitles->setRowCount(arg1);
    for(k = prevRowCount; k < arg1; k++){
        cell = new QTableWidgetItem();
        cell->setTextAlignment(Qt::AlignCenter);
        if(ui->cbUseRowNum->isChecked()){
            cell->setText(QString::number(k+1));
            cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
        else{
            cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
        }
        ui->tblVTitles->setItem(k, 0, cell);
    }
}


void dlgTblEditor::on_edColCount_valueChanged(int arg1)
{
    QTableWidgetItem * cell;
    int k;
    QComboBox * cmbColWidth;
    QComboBox * cmbColFmt;
    QComboBox * cmbEdit;
    int prevColCount = ui->tblValues->columnCount();
    ui->tblValues->setColumnCount(arg1);
    ui->tblHTitles->setColumnCount(arg1);
    ui->tblColFormat->setColumnCount(arg1);
    ui->tblColWidth->setColumnCount(arg1);
    ui->tblColEdit->setColumnCount(arg1);
    for(k = prevColCount; k < arg1; k++){
        cell = new QTableWidgetItem();
        cell->setTextAlignment(Qt::AlignCenter);
        if(ui->cbUseColNum->isChecked()){
            cell->setText(QString::number(k+1));
            cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
        else{
            cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
        }
        ui->tblHTitles->setItem(0, k, cell);
        cmbColWidth = new QComboBox(ui->tblColWidth);
        cmbColWidth->addItem("auto");
        cmbColWidth->addItem("fit");
        cmbColWidth->addItem("1x");
        cmbColWidth->addItem("2x");
        cmbColWidth->addItem("25");
        cmbColWidth->addItem("50");
        cmbColWidth->setToolTip("Column " + QString::number(k + 1));
        cmbColWidth->setEditable(true);
        ui->tblColWidth->setCellWidget(0, k, cmbColWidth);
        cmbColFmt = new QComboBox(ui->tblColWidth);
        cmbColFmt->addItem("char");
        cmbColFmt->addItem("logical");
        cmbColFmt->addItem("numeric");
        cmbColFmt->setEditable(true);
        cmbColFmt->setToolTip("Column " + QString::number(k + 1));
        ui->tblColFormat->setCellWidget(0, k, cmbColFmt);
        cmbEdit = new QComboBox(ui->tblColEdit);
        cmbEdit->addItem("true");
        cmbEdit->addItem("false");
        cmbEdit->setToolTip("Column " + QString::number(k + 1));
        ui->tblColEdit->setCellWidget(0, k, cmbEdit);
    }
}

void dlgTblEditor::on_cbUseColNum_clicked()
{
    QTableWidgetItem * cell;
    int k;
    int colCount = ui->tblHTitles->columnCount();
    for(k = 0; k < colCount; k++){
       cell = new QTableWidgetItem();
       cell->setTextAlignment(Qt::AlignCenter);
       if(ui->cbUseColNum->isChecked()){
           cell->setText(QString::number(k+1));
           cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
       else{
           cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
       }
       ui->tblHTitles->setItem(0, k, cell);
    }
}


void dlgTblEditor::on_cbUseRowNum_clicked()
{
    QTableWidgetItem * cell;
    int k;
    int rowCount = ui->tblVTitles->rowCount();
    for(k = 0; k < rowCount; k++){
        cell = new QTableWidgetItem();
        cell->setTextAlignment(Qt::AlignCenter);
        if(ui->cbUseRowNum->isChecked()){
            cell->setText(QString::number(k+1));
            cell->setFlags(cell->flags() & (~Qt::ItemIsEditable));
        }
        else{
             cell->setFlags(cell->flags() | (Qt::ItemIsEditable));
        }
        ui->tblVTitles->setItem(k, 0, cell);
    }
}

