/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "editabletext.h"
#include <QApplication>
#include "framedlg.h"
#include <QFontMetrics>
#include "mainwnd.h"
#include "ui_mainwnd.h"
#include "painthidden.h"

editableText::editableText(QWidget *parent, abstractUICCtrl *uiCtrl) :
    QLabel(parent)
{    
    underMovement = false;    
    this->parentUICtrl = uiCtrl;
    this->setFocusPolicy(Qt::StrongFocus);
    this->isHidden = false;
}


editableText::editableText(const QString & text, QWidget * parent, abstractUICCtrl *uiCtrl):
    QLabel(text, parent)
{
    QFontMetrics fm(this->font());
    this->resize(fm.boundingRect(this->text()).width(), fm.boundingRect(this->text()).height());
    underMovement = false;    
    this->parentUICtrl=uiCtrl;
    this->setFocusPolicy(Qt::StrongFocus);
    this->isHidden = false;
}

void editableText::setParents(QWidget *parent, abstractUICCtrl *uiCtrl)
{
    setParent(parent);
    this->parentUICtrl = uiCtrl;    
}

void editableText::setHidden(bool isHidden){
    this->isHidden = isHidden;
}

editableText::~editableText()
{

}

int editableText::cordToGrid(int v)
{
    int ret;
    ret = v / 5;
    ret = ret * 5;
    return ret;
}

void editableText::mouseReleaseEvent(QMouseEvent *)
{
    underMovement = false;
    this->parentUICtrl->updateAdjPoints();
}

void editableText::mousePressEvent(QMouseEvent * event)
{
    childWndDlg *wnd = this->parentUICtrl->getParentWnd();
    underMovement = true;
    moveXInit = this->cordToGrid(event->x());
    moveYInit = this->cordToGrid(event->y());

    if(!wnd->isSelected(this->parentUICtrl))
      wnd->addToSelection(this->parentUICtrl);
}

void editableText::mouseMoveEvent(QMouseEvent * event)
{
    QRect sel;
    childWndDlg *wnd = this->parentUICtrl->getParentWnd();
    int dx;
    int dy;
    int pWidth = ((QWidget * )parent())->width();
    int pHeight = ((QWidget * )parent())->height();
    sel = wnd->sizeSelection();
    if(underMovement)
    {
        dx = this->cordToGrid(event->x()) - moveXInit;
        dy = this->cordToGrid(event->y()) - moveYInit;
        if((sel.left() + dx >= 0) &&
           (sel.top() + dy >= 0) &&
           (sel.right() + dx <= pWidth) &&
           (sel.bottom() + dy <= pHeight))
          wnd->moveSelection(dx, dy);
    }
}


void editableText::moveEvent ( QMoveEvent * event )
{
    QLabel::moveEvent(event);
    this->parentUICtrl->updateAdjPoints();
    this->parentWidget()->update();
    mainWnd::getPropPan()->updatePropertiesValues();
}

void editableText::resizeEvent ( QResizeEvent * event )
{
    QLabel::resizeEvent(event);
    this->parentUICtrl->updateAdjPoints();
    this->parentWidget()->update();
    mainWnd::getPropPan()->updatePropertiesValues();
}


void editableText::keyPressEvent(QKeyEvent *e)
{
    if (e->type() == QEvent::KeyPress)
    {
        QKeyEvent* newEvent = new QKeyEvent(QEvent::KeyPress,e->key(), e->modifiers ());
        qApp->postEvent (this->parent(), newEvent, 0);
    }
}

void editableText::mouseDoubleClickEvent(QMouseEvent *event __attribute__ ((unused))){
    callBackEditor * cb;
    if(this->parentUICtrl->callbacks.getCount() > 0){
        if(this->parentUICtrl->getParentWnd()->getTab()){
            this->parentUICtrl->getParentWnd()->getTab()->setCurrentIndex(this->parentUICtrl->getParentWnd()->getTabSrc());

            this->parentUICtrl->getParentWnd()->getSrcWnd()->setCallbackName("callback");
            cb = this->parentUICtrl->callbacks.getCbByName("callback");
            if(cb)
                cb->setActive(true);
            this->parentUICtrl->getParentWnd()->getProPanel()->updatePropertiesValues();
        }
    }
}

void editableText::paintEvent(QPaintEvent *e){
    QLabel::paintEvent(e);
    if(this->isHidden)
        paintHidden ph(this);
}
