/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#ifndef GUIPROJECT_H
#define GUIPROJECT_H

#include "childwnddlg.h"
#include <QObject>
#include <QStringList>
#include <QString>
#include "projectitem.h"
#include <QList>
#include "prjtreewidget.h"
#include "childwnddlg.h"
#include "wdgproject.h"
#include "ui_wdgproject.h"
#include <QFileSystemWatcher>

class guiProject;
class childWndDlg;

class treeItemGroup :  public abstractPrjTreeItem
{
    Q_OBJECT
private:
    guiProject * prj;
public:
    explicit treeItemGroup(guiProject * p, QTreeWidget * parent = 0);
    void activate();
    void populateMenu(QMenu &);
    virtual void selectProject();
    guiProject * getPrj(){return prj;}
private slots:
    void actAddNewDlg(bool checked = false);
    void actAddNewMFile(bool checked = false);
    void actAddNewImage(bool checked = false);
    void actRun(bool checked = false);
    void actRunArguments(bool checked = false);
    void actExport(bool checked = false);
    void actGenPkg(bool checked = false);
    void actProjectProperties(bool checkded = false);
    void actSave(bool checked = false);
    void actClose(bool checked = false);
};

class guiProject:public QMdiSubWindow
{
    Q_OBJECT;
private:
  QString vName;
  QString vPath;
  projectItem* vMainDlgName;
  QString args;

  wdgProject * wdgPrj;

  unsigned vVerA;  // Versión del proyecto:
  unsigned vVerB;  //  vVerA.vVerB.vVerC
  unsigned vVerC;  //

  unsigned fv;     /* File Version: Esto es para saber como debe cargarse el proyecto.
                    * Actualmente hay dos versiones de archivos que representan un proyecto en guiEditor,
                    * a fin de resolver el problema de compatibilidad sobre cómo cargar uno u otro,
                    * se incluye esta propiedad. Por defecto vale 1 para el tipo de archivo original y 2 para
                    * los archivos con el formato 2023.
                   */
  bool vDefBasePath;
  bool vDefImgPath;

  QStringList vSearchPath;
  void copyFolder(QString src, QString dst);

  QFileSystemWatcher *fsw;

protected:
  void closeEvent(QCloseEvent *event);

public:
    QList<projectItem *> items;

    guiProject(QString path, QString name, QWidget *parent);
    ~guiProject();

    QString name();

    QString path();
    QString fullPath();

    bool defBasePath();
    bool defImgPath();

    void setDefBasePath(bool);
    void setDefImgPath(bool);

    QString exportPath();
    QString fcnPath();
    QString guiPath();
    QString imgPath();
    QString otherPath();
    QString pkgPath();
    QString runPath();
    QString libsPath();

    void setVersion(int A, int B, int C);
    void version(int &A, int &B, int &C);

    void setFileVersion(int);
    int fileVersion(void);

    void setMainDlg(QString);
    void setMainDlg(projectItem *);
    projectItem *getMainDlg();

    void setSearchPath(QStringList);
    QStringList searchPath();

    /*void show(QTreeWidgetItem *);*/
    void update(bool openFirst = false);

    void exportPrj();
    void run();
    void setRunArguments(QString args);
    QString runArguments(void);
    void generatePkg();
    void save();
    void disableFSWatch(void);
    void enableFSWatch(void);
    wdgProject * getProjectWidget(){
        return this->wdgPrj;
    }
    QStringList disabledWarningMessages;
    bool disableAllWarnings;
public slots:
    void prjDirChanged(QString);
};

#endif // GUIPROJECT_H
