/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "listctrl.h"
#include "editablelist.h"
#include "storagemanager.h"
#include <QFile>
#include <QTextStream>
#include <QDir>
#include "wdgclrchooser.h"
#include "wdgitemeditor.h"
#include "ui_wdgitemeditor.h"
#include <qdebug.h>
//// Implementación de los métodos de los editores de propiedades


listItemPropEditor::listItemPropEditor(abstractUICCtrl *ctrl):
    abstractPropEditor(ctrl)
{
    this->propName = "String";
}

QString listItemPropEditor::getValue()
{
    int i;
    editableList * list;
    QString ret;
    list = (editableList * )this->ctrl->associatedWidget();
    for(i=0; i < list->count(); i++)
    {
        ret = ret + list->item(i)->text();
        if(i < list->count() - 1)
            ret += "|";
    }
    return ret;
}

bool listItemPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
  //  QRegularExpression r("^\\[[-+]?\\d{1,10}\\s[-+]?\\d{1,10}\\s\\d{1,10}\\s\\d{1,10}\\]$");
    // return newVal.contains(r) == 1;
    return true;
}

void listItemPropEditor::setValue(QString newVal)
{
    QStringList v;
    editableList * list = (editableList * )this->ctrl->associatedWidget();
    int i;
    if(this->isValidValue(newVal))
    {
        list->clear();
        v = newVal.split("|");
        for(i=0; i<v.count(); i++)
            list->addItem(v[i]);
    }
}

QString listItemPropEditor::generateCode()
{
    QString ret;
    ret = "'String', '" + this->getValue() + "'";
    return ret;
}


QWidget * listItemPropEditor::getEditWidget()
{
    wdgItemEditor * wdgId = new wdgItemEditor(this);
    wdgId->setItems(this->getValue());
    wdgId->setToolTip(tr("List of possible options. String of characters separating each value with the pipe character (|)."));
    connect(wdgId->getLEItems(), SIGNAL(editingFinished()), this, SLOT(valueItemChanged()));
    return wdgId;
}

void listItemPropEditor::valueItemChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueItemChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {
        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueItemChanged()));
}

lstBgClrPropEditor::lstBgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "BackgroundColor";
    this->backColor = "[1.000 1.000 1.000]";
}

QString lstBgClrPropEditor::getValue()
{
    return this->backColor;
}

bool lstBgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    //QRegularExpression r("^\\[[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*\\]$");
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void lstBgClrPropEditor::setValue(QString newVal)
{
    QStringList back, fore;
    QString foreStr;
    QString sty;
    int iR, iG, iB;
    double dR, dG, dB;
    QString sR, sG, sB;
    if(this->isValidValue(newVal))
    {
        this->backColor = newVal;
        newVal.remove("[");
        newVal.remove("]");
        foreStr = this->ctrl->properties.getValueByName("ForegroundColor");
        foreStr.remove("[");
        foreStr.remove("]");
        fore = foreStr.split(" ", QString::SkipEmptyParts);
        back = newVal.split(" ", QString::SkipEmptyParts);

        dR = back[0].toDouble()*255;
        iR = (int) dR;
        sR = QString::number(iR);

        dG = back[1].toDouble()*255;
        iG = (int) dG;
        sG = QString::number(iG);

        dB = back[2].toDouble()*255;
        iB = (int) dB;
        sB = QString::number(iB);

        sty = "QListWidget { background-color: rgb(" + sR + ", " + sG + ", " + sB + ");";

        dR = fore[0].toDouble()*255;
        iR = (int) dR;
        sR = QString::number(iR);

        dG = fore[1].toDouble()*255;
        iG = (int) dG;
        sG = QString::number(iG);

        dB = fore[2].toDouble()*255;
        iB = (int) dB;
        sB = QString::number(iB);

        sty = sty + "color: rgb(" + sR + ", " + sG + ", " + sB + ");}";

        this->ctrl->associatedWidget()->setStyleSheet(sty);
    }
}

QString lstBgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'BackgroundColor', " + this->getValue();
    return ret;
}

QWidget * lstBgClrPropEditor::getEditWidget()
{
    wdgClrChooser * wdgCh = new wdgClrChooser(this);
    wdgCh->setColor(this->getValue());
    wdgCh->setToolTip(tr("Background color"));
    connect(wdgCh->getLEColor(), SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    return wdgCh;
}

void lstBgClrPropEditor::valueChooserChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {

        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
}

lstFgClrPropEditor::lstFgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "ForegroundColor";
    this->fgColor = "[0.000 0.000 0.000]";
}

QString lstFgClrPropEditor::getValue()
{
    return this->fgColor;
}

bool lstFgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    //QRegularExpression r("^\\[[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*\\]$");
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void lstFgClrPropEditor::setValue(QString newVal)
{
    QStringList back, fore;
    QString backStr;
    QString sty;
    int iR, iG, iB;
    double dR, dG, dB;
    QString sR, sG, sB;
    if(this->isValidValue(newVal))
    {
        this->fgColor = newVal;
        newVal.remove("[");
        newVal.remove("]");
        backStr = this->ctrl->properties.getValueByName("BackgroundColor");
        backStr.remove("[");
        backStr.remove("]");
        fore = newVal.split(" ", QString::SkipEmptyParts);
        back = backStr.split(" ", QString::SkipEmptyParts);

        dR = back[0].toDouble() * 255;
        iR = (int)dR;
        sR = QString::number(iR);

        dG = back[1].toDouble() * 255;
        iG = (int)dG;
        sG = QString::number(iG);

        dB = back[2].toDouble() * 255;
        iB = (int)dB;
        sB = QString::number(iB);

        sty = "QListWidget { background-color: rgb(" + sR + ", " + sG + ", " + sB + ");";

        dR = fore[0].toDouble() * 255;
        iR = (int)dR;
        sR = QString::number(iR);

        dG = fore[1].toDouble() * 255;
        iG = (int)dG;
        sG = QString::number(iG);

        dB = fore[2].toDouble() * 255;
        iB = (int)dB;
        sB = QString::number(iB);
        sty = sty + "color: rgb(" + sR + ", " + sG + ", " + sB + ");}";

        this->ctrl->associatedWidget()->setStyleSheet(sty);
    }
}

QString lstFgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'ForegroundColor', " + this->getValue();
    return ret;
}

QWidget * lstFgClrPropEditor::getEditWidget()
{
    wdgClrChooser * wdgCh = new wdgClrChooser(this);
    wdgCh->setColor(this->getValue());
    wdgCh->setToolTip(tr("Foreground (text) color"));
    connect(wdgCh->getLEColor(), SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    return wdgCh;
}

void lstFgClrPropEditor::valueChooserChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {

        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
}

unsigned int listCtrl::uicNameCounter = 0;

listCtrl::listCtrl(QWidget *parent, abstractUICCtrl *octaveParent, childWndDlg * parentWnd):abstractUICCtrl(parent, octaveParent, parentWnd)
{
    QStringList defSrc;
    callBackEditor * cb;
    editableList * list = new editableList(parent, this);
    uicNameCounter++;
    this->setCtrlName("ListBox_"+QString::number(uicNameCounter));
    list->addItem("Value A");
    list->addItem("Value B");
    list->addItem("Value C");

    this->setAssociatedWidget(list);
    this->adjTL->asociateCtrl(this->getAssociatedWidget());
    this->adjML->asociateCtrl(this->getAssociatedWidget());
    this->adjBL->asociateCtrl(this->getAssociatedWidget());
    this->adjTR->asociateCtrl(this->getAssociatedWidget());
    this->adjMR->asociateCtrl(this->getAssociatedWidget());
    this->adjBR->asociateCtrl(this->getAssociatedWidget());
    this->adjTM->asociateCtrl(this->getAssociatedWidget());
    this->adjBM->asociateCtrl(this->getAssociatedWidget());


    this->properties.registerProp(new positionPropEditor(this));
    this->properties.registerProp(new fontNamePropEditor(this));
    this->properties.registerProp(new fontSizePropEditor(this));
    this->properties.registerProp(new fontWeightPropEditor(this));
    this->properties.registerProp(new fontAnglePropEditor(this));
    this->properties.registerProp(new toolTipPropEditor(this));
    this->properties.registerProp(new listItemPropEditor(this));
    this->properties.registerProp(new lstBgClrPropEditor(this));
    this->properties.registerProp(new lstFgClrPropEditor(this));
    this->properties.registerProp(new visiblePropEditor(this));
    this->properties.registerProp(new defValuePropEditor(this));
    this->properties.registerProp(new tagPropEditor(this));
    this->properties.registerProp(new userDataPropEditor(this));
    this->properties.registerProp(new vertAlignPropEditor(this));

    cb = new callBackEditor(this, "callback");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% The code added here will be executed when an item is selected from"));
    defSrc.append(QObject::tr("% the list (even if the same value is selected that was already set)"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "buttondownfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This code will be executed when user make  a click over the control"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "deletefcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed immediately before uicontrol is deleted. "));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "keypressfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed when a key was pressed and the control have the focus"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);
}

void listCtrl::setParents(QWidget * parent, abstractUICCtrl * octaveParent, childWndDlg * parentWnd)
{
    editableList * list = dynamic_cast<editableList *>(this->associatedWidget());


    this->adjTL->setParent(parent);
    this->adjML->setParent(parent);
    this->adjBL->setParent(parent);
    this->adjTR->setParent(parent);
    this->adjMR->setParent(parent);
    this->adjBR->setParent(parent);
    this->adjTM->setParent(parent);
    this->adjBM->setParent(parent);

    list->setParents(parent, this);

    this->setOctaveParent(octaveParent);
    this->setParentWnd(parentWnd);
}

QStringList listCtrl::generateMFile(QString path)
{
    QStringList ret;
    QString lineMCode;
    abstractPropEditor * prop;
    int i;
    ret.append("  " + this->properties.getValueByName("Name") + " = uicontrol( ...") ;
    if(getOctaveParent())
      ret.append("\t'parent'," + getOctaveParent()->getCtrlName());
    ret.append("\t'Style','listbox'");
    ret.append("\t'Units', 'pixels'");
    prop = this->properties.getFirst();
    while(prop)
    {
        if(prop->canGenerateCode())
        {
          lineMCode = "\t" + prop->generateCode();
          ret.append(lineMCode);
        }
        prop = this->properties.getNext();
    }
    for(i = 1; i < ret.count() - 1; i++)
        ret[i] = ret[i] + ", ... ";
    ret[i] = ret[i] + ");";
    this->createCallBack(path);
    return ret;
}


abstractUICCtrl * listCtrl::clone(QWidget *parent,
                                  abstractUICCtrl *octaveParent,
                                  childWndDlg * parentWnd){
    callBackEditor * cb;
    callBackEditor *rCb;
    listCtrl * ret = new listCtrl(parent,
                                  octaveParent,
                                  parentWnd);
    abstractPropEditor *prop =  this->properties.getFirst();
    while(prop){
        ret->properties.getPropByName(prop->getPropName())->setValue(prop->getValue());
        prop = this->properties.getNext();
    }
    // La siguiente línea es reemplazada por el fragmento que sigue.
    //ret->srcCallBack = this->srcCallBack;
    cb = this->callbacks.getFirst();
    while(cb){
        rCb = ret->callbacks.getCbByName(cb->getCallBackName());
        /* No puede suceder que no exista un callback con el nombre dado
           porque se trata de un objeto del mismo tipo, pero por seguridad
           se hace la comprobación.
        */
        if(rCb){
          rCb->setActive(cb->isActive());
          rCb->setSrc(cb->getSrcCode());
        }
        cb = this->callbacks.getNext();
    }
    // Fin copia de los callbacks
    return ret;
}

void listCtrl::setHidden(bool isHidden){
    ((editableList *)this->associatedWidget())->setHidden(isHidden);
}


listCtrl::~listCtrl()
{
  delete this->getAssociatedWidget();
}


/// Implementación de la clase generadora
///



listCtrlGen::listCtrlGen():controlGenerator("listCtrl")
{
}

abstractUICCtrl * listCtrlGen::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    listCtrl * list = new listCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = list->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = list->properties.getNext();
    }
    list->deselect();
    return list;

}

listCtrlGenV1::listCtrlGenV1():controlGenerator("listCtrl")
{
}

abstractUICCtrl * listCtrlGenV1::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    QString src;
    listCtrl * list = new listCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = list->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = list->properties.getNext();
    }
    bool cbActive = xml.attributes().value("GenerateCallBack").toString() == "true";
    src = xml.readElementText();
    if(src.length() > 0){
        callBackEditor *cb = list->callbacks.getCbByName("callback");
        if(cb){
            cb->setSrc(src.split('\n'));
            cb->setActive(cbActive);
        }
    }

    list->deselect();
    return list;
}
