/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "mainwnd.h"
#include "ui_mainwnd.h"
#include "framedlg.h"
#include <QLabel>
#include "buttonctrl.h"
#include "radiobuttonctrl.h"
#include "checkboxctrl.h"
#include "editctrl.h"
#include "togglectrl.h"
#include "textctrl.h"
#include "sliderctrl.h"
#include "framectrl.h"
#include "cmbctrl.h"
#include "listctrl.h"
#include "imgctrl.h"
#include "ananoctrl.h"
#include "storagemanager.h"
#include <QFileDialog>
#include <QMdiSubWindow>
#include <QTextStream>
#include <QMessageBox>
#include <QDebug>
#include "wdgdebug.h"
#include "ui_wdgdebug.h"
#include <QFileInfo>
#include "dlgabout.h"
#include "nodeMCUctrl.h"
#include "eduCIAACtrl.h"
#include "childwnddlg.h"
#include "grouppanelctrl.h"
#include "buttongroupctrl.h"
#include "childwndmfile.h"
#include "tablectrl.h"

#include "dlgnewproject.h"
#include "ui_dlgnewproject.h"

#include "childwndtextfile.h"

#include <QStandardPaths>
#include "callbackctrl.h"

#include <QDockWidget>
#include <QHeaderView>

#include <setupwizard.h>
#include <consolelogitem.h>

#include <dlgloadwarning.h>
#include <ui_dlgloadwarning.h>

#include "dlgchecknewversion.h"
#include "ui_dlgchecknewversion.h"

wdgProp *mainWnd::propPanel = NULL;

wdgDebug * mainWnd::debugPanel= NULL;
mainWnd * mainWnd::mainWndPtr = NULL;
dlgPkgGen * mainWnd::pkgWnd = NULL;
configSettings *mainWnd::cfg = NULL;

wdgProp * mainWnd::getPropPan()          { return mainWnd::propPanel;  }
wdgDebug * mainWnd::getDebugWnd()   { return mainWnd::debugPanel;}
mainWnd * mainWnd::getMainWnd()          { return mainWnd::mainWndPtr; }
dlgPkgGen *mainWnd::getPkgGenWnd()      { return mainWnd::pkgWnd;     }

mainWnd::mainWnd(configSettings *appCfg, QWidget *parent) :
  QMainWindow(parent),
  ui(new Ui::mainWnd)
{
  ui->setupUi(this);

  cfg = appCfg;
  mainWndPtr = this;
  setCentralWidget(ui->mdiArea);
  this->setMouseTracking(true);

  /*Creamos los paneles dockeables*/
  dockProp = new QDockWidget(tr("Object inspector"), this);
  dockProp->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  propPanel = new wdgProp(dockProp);
  propPanel->getTblProp()->horizontalHeader()->setStretchLastSection(true);
  propPanel->getTblProp()->setSelectionMode(QTableWidget::NoSelection);
  dockProp->setWidget(propPanel);
  addDockWidget(Qt::RightDockWidgetArea, dockProp);

  dockDbg = new QDockWidget(tr("Octave console"), this);
  this->octaveExec.setParent(this);
  dockDbg->setAllowedAreas(Qt::BottomDockWidgetArea | Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  debugPanel = new wdgDebug(&this->octaveExec, dockDbg);
  dockDbg->setWidget(debugPanel);
  addDockWidget(Qt::BottomDockWidgetArea, dockDbg);

  /*Fin creación paneles dockeables*/

  this->uicState = uicStateNone;
  this->toAdd = NULL;

  this->pkgWnd = new dlgPkgGen(this);
  this->pkgWnd ->setAttribute(Qt::WA_DeleteOnClose, false);
  this->pkgWnd->setWindowFlags(this->pkgWnd->windowFlags() | Qt::WindowStaysOnTopHint);

  connect(&octaveExec, SIGNAL(readyReadStandardError()), debugPanel, SLOT(showStdErr()));
  connect(&octaveExec, SIGNAL(readyReadStandardOutput()), debugPanel, SLOT(showStdOut()));
  QStringList arg;
  if(cfg->octavePathOk() && cfg->tmpDirOk())
    {
      QFileInfo fInfo(cfg->getOctavePath());
      ConsoleLogItem * consoleItem = new ConsoleLogItem(this->getRunningPrj(), this->debugPanel->ui->lstConsoleOut);
      consoleItem->setText(tr("Starting octave..."));
      this->debugPanel->ui->lstConsoleOut->addItem(consoleItem);
      QString cmd = fInfo.path();
      arg << "--path";
      arg << cfg->getTempDir();
      for(int i = 0; i < cfg->getLibPaths().count(); i++)
        {
          arg << "--path";
          arg << cfg->getLibPaths()[i];
        }
      octaveExec.setWorkingDirectory(cmd);

      if (cfg->getQt5PlugginPath().length() > 0)
        {
          QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
          env.insert("QT_PLUGIN_PATH", cfg->getQt5PlugginPath());
          octaveExec.setProcessEnvironment(env);
        }

      octaveExec.start(cfg->getOctavePath(), arg, QIODevice::ReadWrite);

      octaveExec.setTextModeEnabled(true);
      octaveExec.waitForStarted();
      if(octaveExec.isOpen())
        {
          ConsoleLogItem * consoleItem = new ConsoleLogItem(this->getRunningPrj(), this->debugPanel->ui->lstConsoleOut);
          consoleItem->setText(tr("started."));
          this->debugPanel->ui->lstConsoleOut->addItem(consoleItem);

          cmd = "cd " + cfg->getTempDir() + ";\n";
          octaveExec.write(cmd.toLatin1());
        }
    }
  QString snapPath = qgetenv("SNAP");
  if(snapPath.length() == 0){
      if(cfg->getCheckVersion() == cvAsk){
          dlgCheckNewVersion dlg(this);
          if(dlg.exec() == QDialog::Accepted){
                cfg->setCheckVersion(cvCheck);
          }
          else{
                cfg->setCheckVersion(cvDontCheck);
          }
          cfg->saveCfg();
      }

      if(cfg->getCheckVersion() == cvCheck){
          /* Check only once a day */
          QDate qToday = QDate::currentDate();
          unsigned int lastCheck;
          unsigned int today = (qToday.year()*10000) +  (qToday.month()*100) + (qToday.day());
          lastCheck = cfg->getLastVersionCheck();
          if(lastCheck != today){
              cfg->setLastVersionCheck(today);
              cfg->saveCfg();
              vc.doCheck();
          }

      }
  }
  this->showMaximized();
  runningPrj = nullptr;
  updateRecentPrj();
}


mainWnd::~mainWnd()
{    
  //    projectSubWindow* dlg;
  //    QList<QMdiSubWindow *> listDlg;
  //    QList<QMdiSubWindow *>::Iterator itDlg;
  //    listDlg = ui->mdiArea->subWindowList();
  //    for(itDlg = listDlg.begin(); itDlg != listDlg.end(); itDlg++)
  //    {
  //        dlg = dynamic_cast<projectSubWindow *>(*itDlg);
  //        dlg->close();
  //        delete dlg;
  //    }

  this->pkgWnd->close();
  delete this->pkgWnd;

  if(this->octaveExec.isOpen())
    this->octaveExec.close();

  delete ui;
}

void mainWnd::runScript(QString fn, QStringList pathsToAdd, guiProject * prj)
{
  QStringList arg;
  this->runningPrj = prj;
  QString cmd;
  int i;

  if(cfg->octavePathOk() && cfg->tmpDirOk())
    {
      QFileInfo fInfo(cfg->getOctavePath());
      QFileInfo scriptFile(fn);
      if(octaveExec.isOpen())
        octaveExec.close();
      disconnect(&octaveExec, SIGNAL(readyReadStandardError()), debugPanel, SLOT(showStdErr()));
      disconnect(&octaveExec, SIGNAL(readyReadStandardOutput()), debugPanel, SLOT(showStdOut()));


      this->debugPanel->ui->lstConsoleOut->clear();

      connect(&octaveExec, SIGNAL(readyReadStandardError()), debugPanel, SLOT(showStdErr()));
      connect(&octaveExec, SIGNAL(readyReadStandardOutput()), debugPanel, SLOT(showStdOut()));

      ConsoleLogItem * consoleItem = new ConsoleLogItem(this->getRunningPrj(), this->debugPanel->ui->lstConsoleOut);
      consoleItem->setText(tr("Starting octave..."));
      this->debugPanel->ui->lstConsoleOut->addItem(consoleItem);

      cmd = fInfo.path();
      arg << "--path";
      arg << scriptFile.absolutePath();

      for(i = 0; i < pathsToAdd.count(); i++)
        {
          arg << "--path";
          arg << pathsToAdd[i];
        }
      octaveExec.setWorkingDirectory(cmd);

      if (cfg->getQt5PlugginPath().length() > 0)
        {
          QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
          env.insert("QT_PLUGIN_PATH", cfg->getQt5PlugginPath());
          octaveExec.setProcessEnvironment(env);
        }

      octaveExec.start(cfg->getOctavePath(), arg,QIODevice::ReadWrite);
      octaveExec.setTextModeEnabled(true);
      octaveExec.waitForStarted();
      if(octaveExec.isOpen())
        {
          ConsoleLogItem * consoleItem = new ConsoleLogItem(this->getRunningPrj(), this->debugPanel->ui->lstConsoleOut);
          consoleItem->setText(tr("started."));
          this->debugPanel->ui->lstConsoleOut->addItem(consoleItem);
          cmd = "cd '" + scriptFile.absolutePath() + "';\n";
          octaveExec.write(cmd.toLatin1());

          cmd = "runApp;\n";

          octaveExec.write(cmd.toLatin1());
        }
    }
  else
    {
      //Show message falta configurar
      QMessageBox::information(this, tr("Bad configuration"), tr("To run this script is neccesary configure the temporary directory and the octave path from the File/Property menu"));
    }
}

void mainWnd::updateRecentPrj(void){
  int rCount;
  QString prjPath;
  rCount = cfg->getRecentPrjCount();
  if(rCount >= 5){
      prjPath = cfg->getRecentPrj(4);
      ui->actRecentPrj05->setText(prjPath);
      ui->actRecentPrj05->setToolTip(prjPath);
      ui->actRecentPrj05->setVisible(true);
    }
  else{
      ui->actRecentPrj05->setVisible(false);
    }

  if(rCount >= 4){
      prjPath = cfg->getRecentPrj(3);
      ui->actRecentPrj04->setText(prjPath);
      ui->actRecentPrj04->setToolTip(prjPath);
      ui->actRecentPrj04->setVisible(true);
    }
  else{
      ui->actRecentPrj04->setVisible(false);
    }

  if(rCount >= 3){
      prjPath = cfg->getRecentPrj(2);
      ui->actRecentPrj03->setText(prjPath);
      ui->actRecentPrj03->setToolTip(prjPath);
      ui->actRecentPrj03->setVisible(true);
    }
  else{
      ui->actRecentPrj03->setVisible(false);
    }

  if(rCount >= 2){
      prjPath = cfg->getRecentPrj(1);
      ui->actRecentPrj02->setText(prjPath);
      ui->actRecentPrj02->setToolTip(prjPath);
      ui->actRecentPrj02->setVisible(true);
    }
  else{
      ui->actRecentPrj02->setVisible(false);
    }
  if(rCount >= 1){
      prjPath = cfg->getRecentPrj(0);
      ui->actRecentPrj01->setText(prjPath);
      ui->actRecentPrj01->setToolTip(prjPath);
      ui->actRecentPrj01->setVisible(true);
    }
  else{
      ui->actRecentPrj01->setVisible(false);
    }
}

configSettings * mainWnd::getAppConfig()
{
  return cfg;
}

void mainWnd::on_actAddBtn_triggered()
{
  /*
    buttonCtrl * btn;
    frameDlg * frm;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    if(dlg)
    {
        frm = dlg->getFrame();
        btn = new buttonCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = btn;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new buttonCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}


void mainWnd::on_actExit_triggered()
{
  close();
}

void mainWnd::closeEvent(QCloseEvent *e)
{
  QList<QMdiSubWindow *> wndList;
  QList<QMdiSubWindow *>::const_iterator wnd;
  guiProject *prj;
  wndList = ui->mdiArea->subWindowList();
  bool mustClose = true;
  for(wnd = wndList.begin();
      wnd != wndList.end();
      wnd++)
    {
      prj = dynamic_cast<guiProject*>(*wnd);
      mustClose = mustClose && prj->close();
    }
  e->setAccepted(mustClose);
}

void mainWnd::on_actAddLineEdit_triggered()
{
  /*
    editCtrl * edit;
    frameDlg * frm;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    if(dlg)
    {
        frm = dlg->getFrame();
        edit = new editCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = edit;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new editCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddLabel_triggered()
{
  /*
    textCtrl *txt;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        txt = new textCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = txt;
    }
    */
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new textCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actViewPropEdit_triggered()
{
  dockProp->show();
}

void mainWnd::on_actAddCombo_triggered()
{
  /*
    cmbCtrl *cmb;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        cmb = new cmbCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = cmb;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new cmbCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddList_triggered()
{
  /*
    listCtrl *list;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        list = new listCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = list;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new listCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}



void mainWnd::on_actAddCheckBox_triggered()
{
  /*
    checkBoxCtrl *cbCtrl;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        cbCtrl = new checkBoxCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = cbCtrl;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new checkBoxCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddRbBtn_triggered()
{
  /*
    radioButtonCtrl *rbCtrl;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        rbCtrl = new radioButtonCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = rbCtrl;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new radioButtonCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddToggle_triggered()
{
  /*
    toggleCtrl *tggl;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        tggl = new toggleCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = tggl;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new toggleCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddSlider_triggered()
{
  /*
    sliderCtrl *sld;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        sld = new sliderCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = sld;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new sliderCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddImg_triggered()
{
  /*
    imgCtrl *img;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        img = new imgCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = img;
    }
    */
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new imgCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}



void mainWnd::on_actProperties_triggered()
{
  dlgAppConfig * wnd = new dlgAppConfig();
  cfg->paramToWnd(wnd);
  if(wnd->exec())
    {
      cfg->wndToParam(wnd);
      cfg->saveCfg();
    }
  delete wnd;
}


void mainWnd::on_actStop_triggered()
{
  if(octaveExec.isOpen())
    octaveExec.close();
}

void mainWnd::on_actViewOctCon_triggered()
{
  dockDbg->show();
}

void mainWnd::on_actAbout_triggered()
{
  dlgAbout wnd(this);
  wnd.exec();
}

void mainWnd::on_addANano_triggered()
{
  /*
    aNanoCtrl *nano;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        nano = new aNanoCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = nano;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new aNanoCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddNodeMCU_triggered()
{
  /*
    nodeMCUCtrl *node;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        node = new nodeMCUCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = node;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new nodeMCUCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_addEduCIAA_triggered()
{
  /*
    eduCIAACtrl *eduCiaa;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        eduCiaa = new eduCIAACtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = eduCiaa;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new eduCIAACtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}


void mainWnd::on_actAddGroupBox_triggered()
{
  /*
    groupPanelCtrl *gp;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        gp = new groupPanelCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = gp;
    }
    */
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new groupPanelCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}


void mainWnd::on_actNewProject_triggered()
{
  int tabNro;
  QMdiArea * mdiArea;
  wdgSrcEditor * srcEdit;
  QTabWidget * newTab;
  dlgNewProject * newWnd = new dlgNewProject(this);
  if(newWnd->exec() == QDialog::Accepted)
    {
      guiProject * prj = new guiProject(newWnd->ui->leProjectPath->text() + "/" + newWnd->ui->leProjectName->text(), newWnd->ui->leProjectName->text(), this->ui->mdiArea);
      prj->setVersion(newWnd->ui->sbVerA->value(), newWnd->ui->sbVerB->value(), newWnd->ui->sbVerC->value());
      newTab = new QTabWidget(prj->getProjectWidget()->ui->docTabs);
      newTab->setTabPosition(QTabWidget::South);
      tabNro = prj->getProjectWidget()->ui->docTabs->addTab(newTab, "mainDlg");
      prj->getProjectWidget()->ui->docTabs->setCurrentIndex(tabNro);
      srcEdit = new wdgSrcEditor(newTab);
      tabNro = newTab->addTab(srcEdit , tr("[&source]"));
      mdiArea = new QMdiArea(newTab);
      childWndDlg * dlg = new childWndDlg(nullptr,
                                          mainWnd::getPropPan(),
                                          prj);
      mdiArea->addSubWindow(dlg);
      dlg->setSrcEditor(srcEdit);
      dlg->show();
      dlg->setTab(newTab, tabNro);
      tabNro = newTab->addTab(mdiArea, tr("[&design]"));
      newTab->setCurrentIndex(tabNro);

      dlg->getFrame()->setCtrlName("mainDlg");
      dlg->save(prj->guiPath() + QDir::separator() + "mainDlg.xml");

      dlg->setGeometry(1, 1, 250, 100);
      dlg->setWindowModified(false);
      dlg->getFrame()->updateSrcWndCtrls();
      dlg->getFrame()->setCanChangeName(false);
      prj->items.push_back(dlg);
      prj->setMainDlg(dlg);
      prj->getProjectWidget()->updateTreeNames();
      storageManager::saveProject(prj);
      QMdiSubWindow *sub =this->ui->mdiArea->addSubWindow(prj);
      prj->setAttribute(Qt::WA_DeleteOnClose, true);
      sub->showMaximized();
    }
  delete newWnd;
}

void mainWnd::on_actOpenProject_triggered()
{
  dlgLoadWarning dlgWrn(this);
  int ver;
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = QFileDialog::getOpenFileName(this, tr("Open gui project"), this->cfg->getWorkingDir(),tr("GUI Project files (*.prj);;All files (*.*);;"));
  if(!prjName.isNull()){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          ver = storageManager::getProjectVersion(prjName);
          if(ver == 1){
              if(dlgWrn.exec() == QDialog::Accepted){
                  guiProject * prj = storageManager::loadProject(prjName);
                  QMdiSubWindow *sub =this->ui->mdiArea->addSubWindow(prj);
                  prj->setAttribute(Qt::WA_DeleteOnClose, true);
                  sub->showMaximized();
                }
            }
          else{
              guiProject * prj = storageManager::loadProject(prjName);
              QMdiSubWindow *sub =this->ui->mdiArea->addSubWindow(prj);
              prj->setAttribute(Qt::WA_DeleteOnClose, true);
              sub->showMaximized();
            }
        }
      else{
          dlg->setFocus();
        }
    }
}

void mainWnd::on_actRunPrj_triggered()
{
  guiProject *wnd = dynamic_cast<guiProject *>(this->ui->mdiArea->activeSubWindow());
  if(wnd)
    wnd->run();
}

void mainWnd::on_actSaveProject_triggered()
{
  guiProject * prj;
  prj =  dynamic_cast<guiProject *>(this->ui->mdiArea->activeSubWindow());
  if(prj)
    prj->save();
}


void mainWnd::on_actClosePrj_triggered()
{
  guiProject * prj;
  prj =  dynamic_cast<guiProject *>(this->ui->mdiArea->activeSubWindow());
  if(prj)
    prj->close();
}

void mainWnd::on_actAddCallBack_triggered()
{
  /*
    callBackCtrl *cb;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        cb = new callBackCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = cb;
    }*/
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new callBackCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actAddBtnGrp_triggered()
{
  /*
    buttonGroupCtrl *bg;
    childWndDlg *dlg = dynamic_cast<childWndDlg *>(ui->mdiArea->activeSubWindow());
    frameDlg * frm;
    if(dlg)
    {
        frm = dlg->getFrame();
        bg = new buttonGroupCtrl(frm, frm, dlg);
        this->uicState = uicStateAddWidget;
        this->toAdd = bg;
    }
    */
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new buttonGroupCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actRecentPrj01_triggered()
{
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = ui->actRecentPrj01->text();
  if (QFile::exists(prjName)){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          guiProject * prj = storageManager::loadProject(prjName);
          prj->setAttribute(Qt::WA_DeleteOnClose, true);
          this->ui->mdiArea->addSubWindow(prj)->showMaximized();
          this->cfg->registerRecentPrj(prjName);
        }
      else{
          dlg->setFocus();
        }
    }
  else{
      QMessageBox::warning(this, tr("Project not found"), tr("The project \"") + prjName +tr("\" does not exist!"));
    }
}

void mainWnd::on_actRecentPrj02_triggered()
{
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = ui->actRecentPrj02->text();
  if (QFile::exists(prjName)){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          guiProject * prj = storageManager::loadProject(prjName);
          prj->setAttribute(Qt::WA_DeleteOnClose, true);
          this->ui->mdiArea->addSubWindow(prj)->showMaximized();
          this->cfg->registerRecentPrj(prjName);
        }
      else{
          dlg->setFocus();
        }
    }
  else{
      QMessageBox::warning(this, tr("Project not found"), tr("The project \"") + prjName +tr("\" does not exist!"));
    }
}

void mainWnd::on_actRecentPrj03_triggered()
{
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = ui->actRecentPrj03->text();
  if (QFile::exists(prjName)){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          guiProject * prj = storageManager::loadProject(prjName);
          prj->setAttribute(Qt::WA_DeleteOnClose, true);
          this->ui->mdiArea->addSubWindow(prj)->showMaximized();
          this->cfg->registerRecentPrj(prjName);
        }
      else{
          dlg->setFocus();
        }
    }
  else{
      QMessageBox::warning(this, tr("Project not found"), tr("The project \"") + prjName +tr("\" does not exist!"));
    }
}

void mainWnd::on_actRecentPrj04_triggered()
{
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = ui->actRecentPrj04->text();
  if (QFile::exists(prjName)){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          guiProject * prj = storageManager::loadProject(prjName);
          prj->setAttribute(Qt::WA_DeleteOnClose, true);
          this->ui->mdiArea->addSubWindow(prj)->showMaximized();
          this->cfg->registerRecentPrj(prjName);
        }
      else{
          dlg->setFocus();
        }
    }
  else{
      QMessageBox::warning(this, tr("Project not found"), tr("The project \"") + prjName +tr("\" does not exist!"));
    }
}

void mainWnd::on_actRecentPrj05_triggered()
{
  bool isOpen = false;
  guiProject* dlg;
  QList<QMdiSubWindow *> listDlg;
  QList<QMdiSubWindow *>::Iterator itDlg;
  QString prjName = ui->actRecentPrj05->text();
  if (QFile::exists(prjName)){
      listDlg = ui->mdiArea->subWindowList();
      for(itDlg = listDlg.begin(); (itDlg != listDlg.end()) && (!isOpen); itDlg++)
        {
          dlg = dynamic_cast<guiProject *>(*itDlg);
          isOpen = (prjName == dlg->path() + QDir::separator() + dlg->name() + ".prj");
        }
      if(!isOpen){
          guiProject * prj = storageManager::loadProject(prjName);
          prj->setAttribute(Qt::WA_DeleteOnClose, true);
          this->ui->mdiArea->addSubWindow(prj)->showMaximized();
          this->cfg->registerRecentPrj(prjName);
        }
      else{
          dlg->setFocus();
        }
    }
  else{
      QMessageBox::warning(this, tr("Project not found"), tr("The project \"") + prjName +tr("\" does not exist!"));
    }
}

void mainWnd::on_addTable_triggered()
{
  if (this->toAdd)
    delete this->toAdd;
  this->toAdd = new tableCtrl(nullptr, nullptr, nullptr);
  this->uicState = uicStateAddWidget;
}

void mainWnd::on_actConfigWizard_triggered()
{
  setupWizard sw(this->cfg, this);
  sw.exec();
}
