/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "scgetparam.h"
#include "QByteArray"

scGetParam::scGetParam(prgAction *act, uint8_t parId, bool isLast) :
    serialCmd(act, isLast)
{
    buf.clear();
    this->parId = parId;
}

QByteArray scGetParam::dataToSend(void)
{
    QByteArray toSend;
    toSend.push_back(Cmnd_STK_GET_PARAMETER);
    toSend.push_back(this->parId);
    toSend.push_back(Sync_CRC_EOP);
    return toSend;
}

QString scGetParam::cmdInfo(){
    QString ret = "Get Device Info: ";
    ret = ret + QString::number(this->parId , 16);
    return ret;
}

rcvType scGetParam::processResponse(QByteArray data)
{
    rcvType ret = rcvWait;
    buf.push_back(data);
    if ((buf.length() == 3) && (buf[0] == Resp_STK_INSYNC) && (buf[2] == Resp_STK_OK))
    {        
        ret = rcvOk;
    }
    if ((buf.length() == 3) && (buf[0] == Resp_STK_INSYNC) && (buf[2] == Resp_STK_FAILED))
        ret = rcvFail;
    else if((buf.length() == 1) && (buf[0] == Resp_STK_NOSYNC))
        ret = rcvFail;
    return ret;
}

scGetParam::~scGetParam(void)
{

}
