/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "sliderctrl.h"
#include "editableslider.h"
#include "commonproperties.h"
#include <QFile>
#include <QTextStream>
#include <QDir>
#include "wdgclrchooser.h"
#include "storagemanager.h"

sldBgClrPropEditor::sldBgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "BackgroundColor";
}

QString sldBgClrPropEditor::getValue()
{
    QString ret;
    palette = ctrl->associatedWidget()->palette();

    ret = "[";
    ret += QString::number(palette.color(QPalette::Window).redF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::Window).greenF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::Window).blueF(), 'f', 3) + "]";
    return ret;
}

bool sldBgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    //QRegularExpression r("^\\[[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*\\]$");
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void sldBgClrPropEditor::setValue(QString newVal)
{
    QStringList v;
    if(this->isValidValue(newVal))
    {
        newVal.remove("[");
        newVal.remove("]");
        v = newVal.split(" ", QString::SkipEmptyParts);
        palette = this->ctrl->associatedWidget()->palette();
        palette.setColor(QPalette::Window, QColor::fromRgbF(v[0].toFloat(), v[1].toFloat(), v[2].toFloat()));
        this->ctrl->associatedWidget()->setPalette(palette);
    }
}

QString sldBgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'BackgroundColor', " + this->getValue();
    return ret;
}

QWidget * sldBgClrPropEditor::getEditWidget()
{
    wdgClrChooser * wdgCh = new wdgClrChooser(this);
    wdgCh->setColor(this->getValue());
    wdgCh->setToolTip(tr("Background color"));
    connect(wdgCh->getLEColor(), SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    return wdgCh;
}

void sldBgClrPropEditor::valueChooserChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {

        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
}

sldFgClrPropEditor::sldFgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "ForegroundColor";
}

QString sldFgClrPropEditor::getValue()
{
    QString ret;
    palette = ctrl->associatedWidget()->palette();

    ret = "[";
    ret += QString::number(palette.color(QPalette::WindowText).redF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::WindowText).greenF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::WindowText).blueF(), 'f', 3) + "]";
    return ret;
}

bool sldFgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    //QRegularExpression r("^\\[[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*\\]$");
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void sldFgClrPropEditor::setValue(QString newVal)
{
    QStringList v;
    if(this->isValidValue(newVal))
    {
        newVal.remove("[");
        newVal.remove("]");
        v = newVal.split(" ", QString::SkipEmptyParts);
        palette = this->ctrl->associatedWidget()->palette();
        palette.setColor(QPalette::WindowText, QColor::fromRgbF(v[0].toFloat(), v[1].toFloat(), v[2].toFloat()));
        this->ctrl->associatedWidget()->setPalette(palette);
    }
}

QString sldFgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'ForegroundColor', " + this->getValue();
    return ret;
}

QWidget * sldFgClrPropEditor::getEditWidget()
{
    wdgClrChooser * wdgCh = new wdgClrChooser(this);
    wdgCh->setColor(this->getValue());
    wdgCh->setToolTip(tr("Foreground color"));
    connect(wdgCh->getLEColor(), SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    return wdgCh;
}

void sldFgClrPropEditor::valueChooserChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {

        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
}

unsigned int sliderCtrl::uicNameCounter = 0;

unsigned int sliderCtrl::getNameCounter()
{
    return uicNameCounter;
}

sliderCtrl::sliderCtrl(QWidget *parent, abstractUICCtrl *octaveParent, childWndDlg * parentWnd)
    :abstractUICCtrl(parent, octaveParent, parentWnd)
{
    QStringList defSrc;
    callBackEditor * cb;
    uicNameCounter++;
    editableSlider* sld = new editableSlider(parent, this);
    this->setAssociatedWidget(sld);
    this->setCtrlName("Slider_"+QString::number(uicNameCounter));
    sld->resize(60, 25);
    this->adjTL->asociateCtrl(this->getAssociatedWidget());
    this->adjML->asociateCtrl(this->getAssociatedWidget());
    this->adjBL->asociateCtrl(this->getAssociatedWidget());
    this->adjTR->asociateCtrl(this->getAssociatedWidget());
    this->adjMR->asociateCtrl(this->getAssociatedWidget());
    this->adjBR->asociateCtrl(this->getAssociatedWidget());
    this->adjTM->asociateCtrl(this->getAssociatedWidget());
    this->adjBM->asociateCtrl(this->getAssociatedWidget());
    this->properties.registerProp(new positionPropEditor(this));
    this->properties.registerProp(new toolTipPropEditor(this));

    this->properties.registerProp(new valueSliderPropEditor(this));
    this->properties.registerProp(new maxSliderPropEditor(this));
    this->properties.registerProp(new minSliderPropEditor(this));
    this->properties.registerProp(new sldBgClrPropEditor(this));
    this->properties.registerProp(new sldFgClrPropEditor(this));
    this->properties.registerProp(new visiblePropEditor(this));
    this->properties.registerProp(new tagPropEditor(this));
    this->properties.registerProp(new userDataPropEditor(this));
    this->properties.registerProp(new vertAlignPropEditor(this));
    cb = new callBackEditor(this, "callback");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This code will be executed when user change the value of slider."));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);


    defSrc.clear();
    cb = new callBackEditor(this, "buttondownfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This code will be executed when user make  a click over the control"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "deletefcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed immediately before uicontrol is deleted. "));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "keypressfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed when a key was pressed and the control have the focus"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);
}

void sliderCtrl::setParents(QWidget * parent, abstractUICCtrl * octaveParent, childWndDlg * parentWnd)
{
    editableSlider * slider = dynamic_cast<editableSlider *>(this->associatedWidget());


    this->adjTL->setParent(parent);
    this->adjML->setParent(parent);
    this->adjBL->setParent(parent);
    this->adjTR->setParent(parent);
    this->adjMR->setParent(parent);
    this->adjBR->setParent(parent);
    this->adjTM->setParent(parent);
    this->adjBM->setParent(parent);

    slider->setParents(parent, this);

    this->setOctaveParent(octaveParent);
    this->setParentWnd(parentWnd);
}

QStringList sliderCtrl::generateMFile(QString path)
{
    QStringList ret;
    QString lineMCode;
    abstractPropEditor * prop;
    int i;
    ret.append("  " + this->properties.getValueByName("Name") + " = uicontrol( ...") ;
    if(getOctaveParent())
      ret.append("\t'parent'," + getOctaveParent()->getCtrlName());
    ret.append("\t'Style','slider'");
    ret.append("\t'Units', 'pixels'");
    prop = this->properties.getFirst();
    while(prop)
    {
        if(prop->canGenerateCode())
        {
          lineMCode = "\t" + prop->generateCode();
          ret.append(lineMCode);
        }
        prop = this->properties.getNext();
    }
    for(i = 1; i < ret.count() - 1; i++)
        ret[i] = ret[i] + ", ... ";
    ret[i] = ret[i] + ");";
    this->createCallBack(path);
    return ret;
}

abstractUICCtrl * sliderCtrl::clone(QWidget *parent,
                                    abstractUICCtrl *octaveParent,
                                    childWndDlg * parentWnd){
    callBackEditor * cb;
    callBackEditor *rCb;
    sliderCtrl * ret = new sliderCtrl(parent,
                                      octaveParent,
                                      parentWnd);
    abstractPropEditor *prop =  this->properties.getFirst();
    while(prop){
        ret->properties.getPropByName(prop->getPropName())->setValue(prop->getValue());
        prop = this->properties.getNext();
    }
    // La siguiente línea es reemplazada por el fragmento que sigue.
    //ret->srcCallBack = this->srcCallBack;
    cb = this->callbacks.getFirst();
    while(cb){
        rCb = ret->callbacks.getCbByName(cb->getCallBackName());
        /* No puede suceder que no exista un callback con el nombre dado
           porque se trata de un objeto del mismo tipo, pero por seguridad
           se hace la comprobación.
        */
        if(rCb){
          rCb->setActive(cb->isActive());
          rCb->setSrc(cb->getSrcCode());
        }
        cb = this->callbacks.getNext();
    }
    // Fin copia de los callbacks
    return ret;
}

void sliderCtrl::setHidden(bool isHidden){
 ((editableSlider *)this->associatedWidget())->setHiden(isHidden);
}


sliderCtrl::~sliderCtrl()
{
    delete this->getAssociatedWidget();
}

sliderCtrlGen::sliderCtrlGen():controlGenerator("sliderCtrl")
{
}

abstractUICCtrl * sliderCtrlGen::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    sliderCtrl * sld = new sliderCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = sld->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = sld->properties.getNext();
    }
    sld->deselect();
    return sld;
}

sliderCtrlGenV1::sliderCtrlGenV1():controlGenerator("sliderCtrl")
{
}

abstractUICCtrl * sliderCtrlGenV1::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    QString src;
    sliderCtrl * sld = new sliderCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = sld->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = sld->properties.getNext();
    }
    bool cbActive = xml.attributes().value("GenerateCallBack").toString() == "true";
    src = xml.readElementText();
    if(src.length() > 0){
        callBackEditor *cb = sld->callbacks.getCbByName("callback");
        if(cb){
            cb->setSrc(src.split('\n'));
            cb->setActive(cbActive);
        }
    }
    sld->deselect();
    return sld;
}

