﻿/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "tablectrl.h"
#include "editabletable.h"
#include "storagemanager.h"

#include <QFile>
#include <QTextStream>
#include <QDir>
#include "wdgtblbackcolorchooser.h"
#include "wdgclrchooser.h"
#include <QPushButton>
#include "dlgtbleditor.h"
#include "ui_dlgtbleditor.h"
#include <string.h>
#include <QDebug>
#include "mainwnd.h"

tblBgClrPropEditor::tblBgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl){
    this->propName = "BackgroundColor";
    this->firstColor = "[0.9 0.9 0.9]";
    this->secondColor = "[0.8 0.8 0.8]";
    ctrl->associatedWidget()->setAutoFillBackground(true);
}

QString tblBgClrPropEditor::getValue(){
    QString ret, tmp;
    ret = this->firstColor;
    ret.remove("]");
    tmp = this->secondColor;
    tmp.remove("[");
    ret = ret + ";" + tmp;
    return ret;
}

bool tblBgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\;[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void tblBgClrPropEditor::setValue(QString newVal)
{
    QStringList v;
    if(this->isValidValue(newVal)){
        newVal.remove("[");
        newVal.remove("]");
        QStringList parts = newVal.split(";");
        QStringList fColor = parts[0].split(" ");
        QStringList sColor = parts[1].split(" ");
        this->firstColor = "[" + fColor[0] + " " + fColor[1] + " " + fColor[2] + "]";
        this->secondColor = "[" + sColor[0] + " " + sColor[1] + " " + sColor[2] + "]";
    }
}

QString tblBgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'BackgroundColor', " + this->getValue();
    return ret;
}

QWidget * tblBgClrPropEditor::getEditWidget()
{
    wdgTblBackColorChooser * wdgCh = new wdgTblBackColorChooser(this);
    wdgCh->setColors(this->getValue());
    connect(wdgCh->getFirstLEColor(), SIGNAL(editingFinished()), this, SLOT(firstColorChanged()));
    connect(wdgCh->getSecondLEColor(), SIGNAL(editingFinished()), this, SLOT(secondColorChanged()));

    return wdgCh;
}

void tblBgClrPropEditor::firstColorChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(firstColorChanged()));
    if(ed->text().contains(r)){
        this->firstColor = ed->text();
    }
    else{
        ed->setText(this->firstColor);
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(firstColorChanged()));
}

void tblBgClrPropEditor::secondColorChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(secondColorChanged()));
    if(ed->text().contains(r)){
        this->secondColor = ed->text();
    }
    else{
        ed->setText(this->secondColor);
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(secondColorChanged()));
}

tblFgClrPropEditor::tblFgClrPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "ForegroundColor";
}

QString tblFgClrPropEditor::getValue()
{
    QString ret;
    palette = ctrl->associatedWidget()->palette();

    ret = "[";
    ret += QString::number(palette.color(QPalette::WindowText).redF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::WindowText).greenF(), 'f', 3) + " ";
    ret += QString::number(palette.color(QPalette::WindowText).blueF(), 'f', 3) + "]";
    return ret;
}

bool tblFgClrPropEditor::isValidValue(QString newVal __attribute__((unused)))
{
    //QRegularExpression r("^\\[[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*(0?\\.[0-9]+)[[:space:]]*\\]$");
    QRegularExpression r("^\\[[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*((0?\\.[0-9]+)|(1([\\.]?)([0]*)))[[:space:]]*\\]$");
    return newVal.contains(r) == 1;
}

void tblFgClrPropEditor::setValue(QString newVal)
{
    QStringList v;
    if(this->isValidValue(newVal))
    {
        newVal.remove("[");
        newVal.remove("]");
        v = newVal.split(" ", QString::SkipEmptyParts);
        palette = this->ctrl->associatedWidget()->palette();
        palette.setColor(QPalette::WindowText, QColor::fromRgbF(v[0].toFloat(), v[1].toFloat(), v[2].toFloat()));
        this->ctrl->associatedWidget()->setPalette(palette);
    }
}

QString tblFgClrPropEditor::generateCode()
{
    QString ret;
    ret = "'ForegroundColor', " + this->getValue();
    return ret;
}

QWidget * tblFgClrPropEditor::getEditWidget()
{
    wdgClrChooser * wdgCh = new wdgClrChooser(this);
    wdgCh->setColor(this->getValue());
    wdgCh->setToolTip(tr("Foreground (text) color"));
    connect(wdgCh->getLEColor(), SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    return wdgCh;
}

void tblFgClrPropEditor::valueChooserChanged()
{
    QLineEdit * ed;
    ed = (QLineEdit *) QObject::sender();
    disconnect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
    if(this->isValidValue(ed->text()))
    {
        this->setValue(ed->text());
    }
    else
    {

        ed->setText(this->getValue());
    }
    ed->selectAll();
    connect(ed, SIGNAL(editingFinished()), this, SLOT(valueChooserChanged()));
}


tblRearrangePropEditor::tblRearrangePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = false;
  this->propName = "RearrangeableColumns";
}

QString tblRearrangePropEditor::getValue()
{
    QString ret = "off";
    if(value){
        ret = "on";
    }
    return ret;
}

bool tblRearrangePropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "on") ||
                (newVal.toLower() == "off"));
    return ret;
}

void tblRearrangePropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "on")
        this->value = true;
    else
        this->value = false;
}

QWidget * tblRearrangePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Enable (or not) to move columns by clicking and dragging the column headers"));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblRearrangePropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}

tblRowStripingPropEditor::tblRowStripingPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = false;
  this->propName = "RowStriping";
}

QString tblRowStripingPropEditor::getValue()
{
    QString ret = "off";
    if(value){
        ret = "on";
    }
    return ret;
}

bool tblRowStripingPropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "on") ||
                (newVal.toLower() == "off"));
    return ret;
}

void tblRowStripingPropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "on")
        this->value = true;
    else
        this->value = false;
}

QWidget * tblRowStripingPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Setting to indicate whether the table background color will use different colors for each row. "));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblRowStripingPropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}


tblEnablePropEditor::tblEnablePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = true;
  this->propName = "Enable";
}

QString tblEnablePropEditor::getValue()
{
    QString ret = "off";
    if(value){
        ret = "on";
    }
    return ret;
}

bool tblEnablePropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "on") ||
                (newVal.toLower() == "off"));
    return ret;
}

void tblEnablePropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "on")
        this->value = true;
    else
        this->value = false;
}

QWidget * tblEnablePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Sets whether this table object is active or is grayed out."));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblEnablePropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}

tblSelHighlightPropEditor::tblSelHighlightPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = true;
  this->propName = "SelectionHighlight";
}

QString tblSelHighlightPropEditor::getValue()
{
    QString ret = "off";
    if(value){
        ret = "on";
    }
    return ret;
}

bool tblSelHighlightPropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "on") ||
                (newVal.toLower() == "off"));
    return ret;
}

void tblSelHighlightPropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "on")
        this->value = true;
    else
        this->value = false;
}

QWidget * tblSelHighlightPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("If selectionhighlight is \"on\", then the uitable’s selection state is visually highlighted."));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblSelHighlightPropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}


tblPickablePartsPropEditor::tblPickablePartsPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = tblPPVisible;
  this->propName = "PickableParts";
}

QString tblPickablePartsPropEditor::getValue()
{
    QString ret;
    switch(this->value)
    {
        case tblPPAll:
          ret = "all";
          break;
        case tblPPNone:
          ret = "none";
          break;
        case tblPPVisible:
          ret = "visible";
    }
    return ret;
}

bool tblPickablePartsPropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "all") ||
                (newVal.toLower() == "none") ||
                (newVal.toLower() == "visible"));
    return ret;
}

void tblPickablePartsPropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "all")
        this->value = tblPPAll;
    else if(newVal.toLower() == "none")
        this->value = tblPPNone;
    else if(newVal.toLower() == "visible")
        this->value = tblPPVisible;
}

QWidget * tblPickablePartsPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("all");
    cb->addItem("none");
    cb->addItem("visible");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Specify whether uitable will accept mouse clicks."));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblPickablePartsPropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}

tblHittestPropEditor::tblHittestPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
  this->value = true;
  this->propName = "HitTest";
}

QString tblHittestPropEditor::getValue()
{
    QString ret = "off";
    if(value){
        ret = "on";
    }
    return ret;
}

bool tblHittestPropEditor::isValidValue(QString newVal)
{
    bool ret = ((newVal.toLower() == "on") ||
                (newVal.toLower() == "off"));
    return ret;
}

void tblHittestPropEditor::setValue(QString newVal)
{
    if(newVal.toLower() == "on")
        this->value = true;
    else
        this->value = false;
}

QWidget * tblHittestPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("on");
    cb->addItem("off");
    cb->setCurrentIndex(cb->findText(this->getValue()));
    cb->setToolTip(tr("Specify whether uitable processes mouse events or passes them to ancestors of the object."));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    return cb;
}

void tblHittestPropEditor::valueChooserChanged(QString text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
        this->setValue(text);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(valueChooserChanged(QString)));
    }
}

tblDataConfig::tblDataConfig(QString cfg){
    int f, c, k, cnt;
    int minF, minC;
    QString key;
    QString val;
    QStringList cfgOptions = cfg.split("|");
    for(cnt = 0; cnt < cfgOptions.count(); cnt++){
        QStringList cfgKeyVal = cfgOptions[cnt].split("=");
        if(cfgKeyVal.count() == 2){
            key = cfgKeyVal[0].trimmed();
            val = cfgKeyVal[1].trimmed();
            if(key == "colCount"){
                this->colCount = val.toInt();
            }
            else if(key == "rowCount"){
                this->rowCount = val.toInt();
            }
            else if(key == "defineWith"){
                this->baseDef = val;
            }
            else if(key == "rowNames"){
                if(val == "numbered"){
                    this->useNumberedRow = true;
                    for(k = 0; k < this->rowCount; k++){
                        this->rowTitle.append(QString::number(k + 1));
                    }
                }
                else{
                    this->useNumberedRow = false;
                    this->rowTitle = val.split(",");
                    for(k = 0; k < this->rowTitle.count(); k++){
                        this->rowTitle[k] = this->rowTitle[k].trimmed();
                    }
                }
            }
            else if(key == "columnNames"){
                if(val == "numbered"){
                    this->useNumberedCol = true;
                    for(k = 0; k < this->colCount; k++){
                        this->colTitle.append(QString::number(k + 1));
                    }
                }
                else{
                    this->useNumberedCol = false;
                    this->colTitle = val.split(",");
                    for(k = 0; k < this->colTitle.count(); k++){
                        this->colTitle[k] = this->colTitle[k].trimmed();
                    }
                }
            }
            else if(key == "columnWidth"){
                this->colWidth = val.split(",");
                for(k = 0; k < this->colWidth.count(); k++){
                    this->colWidth[k] = this->colWidth[k].trimmed();
                }
            }
            else if(key == "columnFormat"){
                this->colFmt = val.split(",");
                for(k = 0; k < this->colFmt.count(); k++){
                    this->colFmt[k] = this->colFmt[k].trimmed();
                }
            }
            else if(key == "columneditable"){
                this->colEdit = val.split(",");
                for(k = 0; k < this->colEdit.count(); k++){
                    this->colEdit[k] = this->colEdit[k].trimmed();
                }
            }
            else if(key == "data"){

                data = new char**[this->rowCount];
                for(f = 0; f < this->rowCount; f++)
                    data[f] = new char*[this->colCount];
                QStringList rData = val.split(";");
                minF = this->rowCount;
                if(rData.count() < minF)
                    minF = rData.count();
                for(f = 0; f < minF; f++){
                    QStringList cData = rData[f].split(",");
                    minC = this->colCount;
                    if(cData.count() < minC)
                        minC = cData.count();
                    for(c = 0; c < minC; c++){
                        data[f][c] = new char[cData[c].length()];
                        strcpy(data[f][c], cData[c].toStdString().c_str());
                    }
                }
            }
        }
    }
}

int tblDataConfig::getRowCount(void){
    return this->rowCount;
}

int tblDataConfig::getColCount(void){
    return this->colCount;
}

QString tblDataConfig::getColFmt(int inx){
    QString ret = "";
    if((inx >= 0) && (inx < this->colFmt.count()))
        ret = this->colFmt[inx];
    return ret;
}

QString tblDataConfig::getColWidth(int inx){
    QString ret = "";
    if((inx >= 0) && (inx < this->colWidth.count()))
        ret = this->colWidth[inx];
    return ret;
}

QString tblDataConfig::getRowTitle(int f){
    QString ret = "";
    if((f >= 0) && (f < this->rowTitle.count()))
        ret = this->rowTitle[f];
    return ret;
}

QString tblDataConfig::getColTitle(int c){
    QString ret = "";
    if((c >= 0) && (c < this->colTitle.count()))
        ret = this->colTitle[c];
    return ret;
}

QString tblDataConfig::getColEdit(int c){
    QString ret = "";
    if((c >= 0) && (c < this->colEdit.count()))
        ret = this->colEdit[c];
    return ret;
}

QString tblDataConfig::getData(int f, int c){
    QString ret;
    if((f >= 0) && (c >= 0) && (f < rowCount) && (c < colCount)){
        ret = QString(data[f][c]);
    }
    else{
        ret = "";
    }
    return ret;
}

QString tblDataConfig::getBaseDef(void){
    return this->baseDef;
}

bool tblDataConfig::getNumberedRows(void){
    return this->useNumberedRow;
}
bool tblDataConfig::getNumberedCols(void){
    return this->useNumberedCol;
}

tblDataConfig::~tblDataConfig(void){
    int f, c;
    for(f = 0; f < this->rowCount; f++){
        for(c = 0; c < this->colCount; c++){
            delete []data[f][c];
        }
    }
    for(f = 0; f < this->rowCount; f++){
        delete []data[f];
    }
    delete []data;
}

updButton::updButton(abstractPropEditor * prop):updPropWdg(prop){

}

void updButton::updPropValue(){

}

QString tblDataPropEditor::genColWidth(tblDataConfig &cfg){
    QString ret;
    int k;
    bool isInt;
    ret ="'columnwidth', {";
    for(k = 0; k < cfg.getColCount(); k++){
        cfg.getColWidth(k).toInt(&isInt);
        if(isInt)
            ret = ret + cfg.getColWidth(k) ;
        else
          ret = ret + "'" + cfg.getColWidth(k) + "'";
        if(k < cfg.getColCount() - 1)
            ret = ret + ", ";
    }
    ret = ret + "}";
    return ret;
}

QString tblDataPropEditor::genColFomat(tblDataConfig &cfg){
    QString ret;
    int k;
    ret = "'columnFormat', {";
    for(k = 0; k < cfg.getColCount(); k++){
        if((cfg.getColFmt(k).startsWith("[") || cfg.getColFmt(k).startsWith("{")) &&
           (cfg.getColFmt(k).endsWith("]") || cfg.getColFmt(k).endsWith("}")))
                ret += cfg.getColFmt(k);
        else
            ret += "'" + cfg.getColFmt(k) + "'";
        if (k < cfg.getColCount() - 1)
            ret += ", ";
    }
    ret = ret + "}";
    return ret;
}

QString tblDataPropEditor::genColNames(tblDataConfig &cfg){
    QString ret;
    int k;
    ret = "'columnName',";
    if(cfg.getNumberedCols()){
        ret = ret + "'numbered'";
    }
    else{
        ret = ret + "{";
        for(k = 0; k < cfg.getColCount(); k++){
            ret = ret + "'" + cfg.getColTitle(k) + "'";
            if(k < cfg.getColCount() - 1)
                ret = ret + ", ";
        }
        ret = ret + "}";
    }

    return ret;
}

QString tblDataPropEditor::genColEditable(tblDataConfig &cfg){
    QString ret;
    int k;
    ret = "'columneditable', [";
    for(k = 0; k < cfg.getColCount(); k++){
        ret = ret + cfg.getColEdit(k) ;
        if(k < cfg.getColCount() - 1)
            ret = ret + ", ";
    }
    ret = ret + "]";
    return ret;
}

QString tblDataPropEditor::genRowNames(tblDataConfig &cfg){
    QString ret;
    int k;
    ret = "'rowName',";
    if(cfg.getNumberedRows()){
        ret = ret + "'numbered'";
    }
    else{
        ret = ret + "{";
        for(k = 0; k < cfg.getRowCount(); k++){
            ret = ret + "'" + cfg.getRowTitle(k) + "'";
            if(k < cfg.getRowCount() - 1)
                ret = ret + ", ";
        }
        ret = ret + "}";
    }
    return ret;
}

QString tblDataPropEditor::genData(tblDataConfig &cfg){
    QString ret;
    QString def;
    int f, c;

    def = cfg.getBaseDef();
    ret = "'data', ";
    if(def == "Array")
        ret = ret + "[";
    else
        ret = ret + "{";
    for(f = 0; f < cfg.getRowCount(); f++){
        for(c = 0; c < cfg.getColCount(); c++){
            if(cfg.getColFmt(c).toLower() == "char"){
                ret = ret + "'";
            }
            ret = ret + cfg.getData(f, c);
            if(cfg.getColFmt(c).toLower() == "char"){
                ret = ret + "'";
            }
            if(c < cfg.getColCount() - 1){
                ret = ret + ",";
            }
        }
        if(f < cfg.getRowCount() - 1)
            ret = ret + ";";
    }
    if(def == "Array")
        ret = ret + "]";
    else
        ret = ret + "}";
    return ret;
}


tblDataPropEditor::tblDataPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl){
    this->setValue("colCount = 2 | rowCount = 2 | defineWith = Cell | rowNames = numbered | columnNames = numbered | columnWidth = auto,auto | columnFormat = numeric,numeric | columneditable = true,true | data = 1,2;3,4");
    this->propName = "data";
}

QString tblDataPropEditor::getValue(){
    return this->value;
}

bool tblDataPropEditor::isValidValue(QString newVal){
    return newVal.length() > 0;
}

void tblDataPropEditor::setValue(QString newVal){
    QTableWidgetItem *cell;
    int f, c;
    if (newVal.length() > 0){
        this->value = newVal;
        tblDataConfig tblCfg(newVal);
        QTableWidget *tbl = (QTableWidget *)this->ctrl->associatedWidget();
        tbl->setColumnCount(tblCfg.getColCount());
        tbl->setRowCount(tblCfg.getRowCount());
        /* set table data */
        for(f = 0; f < tblCfg.getRowCount(); f++){
            for(c = 0; c < tblCfg.getColCount(); c++){
                cell = new QTableWidgetItem (tblCfg.getData(f, c));
                tbl->setItem(f, c, cell);
            }
        }
        /* set */
        for(f = 0; f < tblCfg.getRowCount(); f++){
            cell = new QTableWidgetItem(tblCfg.getRowTitle(f));
            cell->setTextAlignment(Qt::AlignCenter);
            tbl->setVerticalHeaderItem(f, cell);
        }
        for(c = 0; c < tblCfg.getColCount(); c++){
            cell = new QTableWidgetItem(tblCfg.getColTitle(c));
            cell->setTextAlignment(Qt::AlignCenter);
            tbl->setHorizontalHeaderItem(c, cell);
        }
    }
}

QWidget * tblDataPropEditor::getEditWidget(){
    updButton * btn = new updButton(this);
    btn->setToolTip(tr("Table data and format setup"));
    btn->setText("...");
    connect(btn, SIGNAL(clicked(bool)), this, SLOT(clicked(bool)));
    return btn;
}

QString tblDataPropEditor::generateCode()
{
    QString ret;
    tblDataConfig tblCfg(this->value);
    ret =         this->genColWidth(tblCfg)    + ", ...\n";
    ret += "\t" + this->genColFomat(tblCfg)    + ", ...\n";
    ret += "\t" + this->genColNames(tblCfg)    + ", ...\n";
    ret += "\t" + this->genColEditable(tblCfg) + ", ...\n";
    ret += "\t" + this->genRowNames(tblCfg)    + ", ...\n";
    ret += "\t" + this->genData(tblCfg);
    return ret;
}

void tblDataPropEditor::clicked(bool checked __attribute__((unused))){
   QString tblDef;
    dlgTblEditor *tblEd = new dlgTblEditor(this->value);
   if (tblEd->exec() == QDialog::Accepted){
       tblDef = tblEd->getValue();
       this->setValue(tblDef);
   }
   delete tblEd;
}



tblFontNamePropEditor::tblFontNamePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontName";
    this->ctrl->associatedWidget()->setFont(mainWnd::getMainWnd()->getAppConfig()->getCtrlFont());
}

QString tblFontNamePropEditor::getValue()
{
    return this->ctrl->associatedWidget()->font().family();
}

bool tblFontNamePropEditor::isValidValue(QString newVal)
{
    return fontDB.families().contains(newVal);
}

void tblFontNamePropEditor::setValue(QString newVal)
{
    QFont font;
    int f, c;
    QTableWidgetItem * cell;
    QTableWidget *tbl;
    font = this->ctrl->associatedWidget()->font();
    font.setFamily(newVal);
    this->ctrl->associatedWidget()->setFont(font);
    tbl = (QTableWidget *)this->ctrl->associatedWidget();
    for(f = 0; f < tbl->rowCount(); f++){
        for(c = 0; c < tbl->columnCount(); c++){
            cell = tbl->item(f, c);
            if(cell){
                cell->setFont(font);
            }
            cell = tbl->horizontalHeaderItem(c);
            if(cell){
                cell->setFont(font);
            }
        }
        cell = tbl->verticalHeaderItem(f);
        if(cell){
            cell->setFont(font);
        }
    }
    tbl->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

QString tblFontNamePropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * tblFontNamePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItems(fontDB.families());
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
    cb->setToolTip(tr("Sets the font to use in the control. Keep in mind that if you use an uncommon font, it may not be available on other computers or operating systems."));
    return cb;
}


void tblFontNamePropEditor::fontChanged(const QString & txt)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(txt))
        this->setValue(txt);
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontChanged(QString)));
    }
}

tblFontSizePropEditor::tblFontSizePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontSize";
}

QString tblFontSizePropEditor::getValue()
{
    return QString::number(this->ctrl->associatedWidget()->font().pointSize());
}

bool tblFontSizePropEditor::isValidValue(QString newVal)
{
    bool Ok;
    double dVal;
    dVal = newVal.toInt(&Ok);
    return (dVal > 0) && Ok;
}

void tblFontSizePropEditor::setValue(QString newVal)
{
    int f, c;
    QTableWidgetItem * cell;
    QTableWidget *tbl;
    QFont font = this->ctrl->associatedWidget()->font();
    font.setPointSize(newVal.toInt());
    this->ctrl->associatedWidget()->setFont(font);

    tbl = (QTableWidget *)this->ctrl->associatedWidget();
    for(f = 0; f < tbl->rowCount(); f++){
        for(c = 0; c < tbl->columnCount(); c++){
            cell = tbl->item(f, c);
            if(cell){
                cell->setFont(font);
            }
            cell = tbl->horizontalHeaderItem(c);
            if(cell){
                cell->setFont(font);
            }
        }
        cell = tbl->verticalHeaderItem(f);
        if(cell){
            cell->setFont(font);
        }
    }
    tbl->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);

}

QString tblFontSizePropEditor::generateCode()
{
    QString ret;

    ret = "'" + this->propName + "', " + this->getValue() + ", ";
    ret += "'FontUnits', 'points'";
    return ret;
}

QWidget * tblFontSizePropEditor::getEditWidget()
{
    spinUpdPropWdg * sb = new spinUpdPropWdg(this);
    sb->setAlignment(Qt::AlignRight);
    sb->setMinimum(1);
    sb->setMaximum(5000);
    sb->setSingleStep(1);
    sb->setValue(this->getValue().toInt());
    sb->setToolTip(tr("Set font size using points as unit"));
    connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
    return sb;
}

void tblFontSizePropEditor::fontSizeChanged(const QString &text)
{
    spinUpdPropWdg * sb = (spinUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
        this->setValue(text);
    }
    else
    {
        disconnect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
        sb->setValue(this->getValue().toInt());
        connect(sb, SIGNAL(valueChanged(QString)), this, SLOT(fontSizeChanged(QString)));
    }
}

tblFontWeightPropEditor::tblFontWeightPropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontWeight";
}

QString tblFontWeightPropEditor::getValue()
{
    QString ret;
    ret = "normal";
    if(this->ctrl->associatedWidget()->font().bold())
        ret = "bold";
    return ret;
}

bool tblFontWeightPropEditor::isValidValue(QString newVal)
{
    return (newVal == "bold") || (newVal == "normal");
}

void tblFontWeightPropEditor::setValue(QString newVal)
{
    int f, c;
    QTableWidgetItem * cell;
    QTableWidget *tbl;
    QFont font = this->ctrl->associatedWidget()->font();
    font.setBold(newVal == "bold");
    this->ctrl->associatedWidget()->setFont(font);
    tbl = (QTableWidget *)this->ctrl->associatedWidget();
    for(f = 0; f < tbl->rowCount(); f++){
        for(c = 0; c < tbl->columnCount(); c++){
            cell = tbl->item(f, c);
            if(cell){
                cell->setFont(font);
            }
            cell = tbl->horizontalHeaderItem(c);
            if(cell){
                cell->setFont(font);
            }
        }
        cell = tbl->verticalHeaderItem(f);
        if(cell){
            cell->setFont(font);
        }
    }
    tbl->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

QString tblFontWeightPropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * tblFontWeightPropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("normal");
    cb->addItem("bold");
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
    cb->setToolTip(tr("Sets bold attribute to font used in control"));
    return cb;
}

void tblFontWeightPropEditor::fontWeightChanged(const QString & text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
      this->setValue(text);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontWeightChanged(QString)));
    }
}
//////////////////
tblFontAnglePropEditor::tblFontAnglePropEditor(abstractUICCtrl *ctrl):abstractPropEditor(ctrl)
{
    this->propName = "FontAngle";
}

QString tblFontAnglePropEditor::getValue()
{
    QString ret;
    ret = "normal";
    if(this->ctrl->associatedWidget()->font().italic())
        ret = "italic";
    return ret;
}

bool tblFontAnglePropEditor::isValidValue(QString newVal)
{
    return (newVal == "italic") || (newVal == "normal");
}

void tblFontAnglePropEditor::setValue(QString newVal)
{
    int f, c;
    QTableWidgetItem * cell;
    QTableWidget *tbl;
    QFont font = this->ctrl->associatedWidget()->font();
    font.setItalic(newVal == "italic");
    this->ctrl->associatedWidget()->setFont(font);
    tbl = (QTableWidget *)this->ctrl->associatedWidget();
    for(f = 0; f < tbl->rowCount(); f++){
        for(c = 0; c < tbl->columnCount(); c++){
            cell = tbl->item(f, c);
            if(cell){
                cell->setFont(font);
            }
            cell = tbl->horizontalHeaderItem(c);
            if(cell){
                cell->setFont(font);
            }
        }
        cell = tbl->verticalHeaderItem(f);
        if(cell){
            cell->setFont(font);
        }
    }
    tbl->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

QString tblFontAnglePropEditor::generateCode()
{
    return "'" + this->propName + "', '" + this->getValue() + "'";
}

QWidget * tblFontAnglePropEditor::getEditWidget()
{
    comboUpdPropWdg * cb = new comboUpdPropWdg(this);
    cb->addItem("normal");
    cb->addItem("italic");
    //cb->setCurrentText(this->getValue());
    cb->setCurrentIndex(cb->findText(this->getValue()));
    connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
    cb->setToolTip(tr("Sets italic attribute to font used in control"));
    return cb;
}

void tblFontAnglePropEditor::fontAngleChanged(const QString & text)
{
    comboUpdPropWdg * cb = (comboUpdPropWdg *)QObject::sender();
    if(this->isValidValue(text))
    {
      this->setValue(text);
    }
    else
    {
        disconnect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
        cb->setCurrentIndex(cb->findText(this->getValue()));
        connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontAngleChanged(QString)));
    }
}

unsigned int tableCtrl::uicNameCounter = 0;

unsigned int tableCtrl::getNameCounter()
{
    return uicNameCounter;
}

tableCtrl::tableCtrl(QWidget *parent, abstractUICCtrl *octaveParent, childWndDlg * parentWndDlg):abstractUICCtrl(parent, octaveParent, parentWndDlg)
{
    QStringList defSrc;
    callBackEditor * cb;
    editableTable * table = new editableTable(parent, this);
    uicNameCounter++;
    this->setCtrlName("Table_"+QString::number(uicNameCounter));       

    this->setAssociatedWidget(table);
    this->adjTL->asociateCtrl(this->getAssociatedWidget());
    this->adjML->asociateCtrl(this->getAssociatedWidget());
    this->adjBL->asociateCtrl(this->getAssociatedWidget());
    this->adjTR->asociateCtrl(this->getAssociatedWidget());
    this->adjMR->asociateCtrl(this->getAssociatedWidget());
    this->adjBR->asociateCtrl(this->getAssociatedWidget());
    this->adjTM->asociateCtrl(this->getAssociatedWidget());
    this->adjBM->asociateCtrl(this->getAssociatedWidget());
    this->properties.registerProp(new tblFontNamePropEditor(this));
    this->properties.registerProp(new tblFontSizePropEditor(this));
    this->properties.registerProp(new tblFontWeightPropEditor(this));
    this->properties.registerProp(new tblFontAnglePropEditor(this));
    this->properties.registerProp(new toolTipPropEditor(this));
    this->properties.registerProp(new positionPropEditor(this));
    this->properties.registerProp(new visiblePropEditor(this));
    this->properties.registerProp(new tblFgClrPropEditor(this));
    this->properties.registerProp(new tblBgClrPropEditor(this));
    this->properties.registerProp(new tblRearrangePropEditor(this));
    this->properties.registerProp(new tblRowStripingPropEditor(this));
    this->properties.registerProp(new tblEnablePropEditor(this));
    this->properties.registerProp(new tblSelHighlightPropEditor(this));
    this->properties.registerProp(new tblPickablePartsPropEditor(this));
    this->properties.registerProp(new tblHittestPropEditor(this));
    this->properties.registerProp(new tagPropEditor(this));
    this->properties.registerProp(new userDataPropEditor(this));
    this->properties.registerProp(new tblDataPropEditor(this));
    defSrc.clear();
    cb = new callBackEditor(this, "buttondownfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This code will be executed when user make  a click over the control"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);


    defSrc.clear();
    cb = new callBackEditor(this, "celleditcallback");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This callback will be executed when a cell was edited"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "cellselectioncallback");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% This callback will be executed when a cell was selected"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "deletefcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed immediately before uicontrol is deleted. "));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "keypressfcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed when a key was pressed and the control have the focus"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

    defSrc.clear();
    cb = new callBackEditor(this, "keyreleasefcn");
    defSrc.append("% ");
    defSrc.append(QObject::tr("% Callback function executed when a key was released"));
    defSrc.append("% ");
    defSrc.append("");
    cb->setDefSrc(defSrc);
    this->callbacks.registerCB(cb);

}
void tableCtrl::setParents(QWidget * parent, abstractUICCtrl * octaveParent, childWndDlg * parentWnd)
{
    editableTable * table = dynamic_cast<editableTable *>(this->associatedWidget());
    table->setParents(parent, this);

    this->adjTL->setParent(parent);
    this->adjML->setParent(parent);
    this->adjBL->setParent(parent);
    this->adjTR->setParent(parent);
    this->adjMR->setParent(parent);
    this->adjBR->setParent(parent);
    this->adjTM->setParent(parent);
    this->adjBM->setParent(parent);

    this->setOctaveParent(octaveParent);
    this->setParentWnd(parentWnd);
}

QStringList tableCtrl::generateMFile(QString path)
{
    QStringList ret;
    QString lineMCode;
    abstractPropEditor * prop;
    int i;
    ret.append("  " + this->properties.getValueByName("Name") + " = uitable( ...") ;
    if(getOctaveParent())
      ret.append("\t'parent'," + getOctaveParent()->getCtrlName());
    ret.append("\t'Units', 'pixels'");
    prop = this->properties.getFirst();
    while(prop)
    {
        if(prop->canGenerateCode())
        {
          lineMCode = "\t" + prop->generateCode();
          ret.append(lineMCode);
        }
        prop = this->properties.getNext();
    }
    for(i = 1; i < ret.count() - 1; i++)
        ret[i] = ret[i] + ", ... ";
    ret[i] = ret[i] + ");";
    this->createCallBack(path);
    return ret;
}

abstractUICCtrl * tableCtrl::clone(QWidget *parent,
                                  abstractUICCtrl *octaveParent,
                                  childWndDlg * parentWnd){
    callBackEditor * cb;
    callBackEditor *rCb;
    tableCtrl * ret = new tableCtrl(parent,
                                  octaveParent,
                                  parentWnd);
    abstractPropEditor *prop =  this->properties.getFirst();
    while(prop){
        ret->properties.getPropByName(prop->getPropName())->setValue(prop->getValue());
        prop = this->properties.getNext();
    }
    // La siguiente línea es reemplazada por el fragmento que sigue.
    //ret->srcCallBack = this->srcCallBack;
    cb = this->callbacks.getFirst();
    while(cb){
        rCb = ret->callbacks.getCbByName(cb->getCallBackName());
        /* No puede suceder que no exista un callback con el nombre dado
           porque se trata de un objeto del mismo tipo, pero por seguridad
           se hace la comprobación.
        */
        if(rCb){
          rCb->setActive(cb->isActive());
          rCb->setSrc(cb->getSrcCode());
        }
        cb = this->callbacks.getNext();
    }
    // Fin copia de los callbacks
    return ret;
}

void tableCtrl::setHidden(bool isHidden){
    ((editableTable *)this->associatedWidget())->setHidden(isHidden);
}

tableCtrl::~tableCtrl()
{
  delete this->getAssociatedWidget();
}

tableCtrlGen::tableCtrlGen():controlGenerator("tableCtrl")
{
}

abstractUICCtrl * tableCtrlGen::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    tableCtrl * txt = new tableCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = txt->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = txt->properties.getNext();
    }
    txt->deselect();
    return txt;
}

tableCtrlGenV1::tableCtrlGenV1():controlGenerator("tableCtrl")
{
}

abstractUICCtrl * tableCtrlGenV1::getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg)
{
    tableCtrl * txt = new tableCtrl(parent->getAssociatedWidget(), parent, dlg);
    abstractPropEditor * prop;
    prop = txt->properties.getFirst();
    while(prop)
    {
      prop->setValue(xml.attributes().value(prop->getPropName()).toString());
      prop = txt->properties.getNext();
    }
    /** El control uitable no tiene un evento predeterminado asociado
      * a la propiedad callback
      * Por esto, si bien se lee el posible código asociado, no se utiliza.
    **/
    xml.readElementText();
    txt->deselect();
    return txt;
}
