/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#ifndef TABLECTRL_H
#define TABLECTRL_H
#include <QWidget>
#include "abstractuicctrl.h"
#include <QTableWidget>
#include <QFontMetrics>
#include "commonproperties.h"
#include "controlgenerator.h"
#include <QPushButton>

class tblBgClrPropEditor: public abstractPropEditor
{
 Q_OBJECT
private:
    QString firstColor;
    QString secondColor;
public:
    tblBgClrPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void firstColorChanged();
    virtual void secondColorChanged();
};

class tblFgClrPropEditor: public abstractPropEditor
{
 Q_OBJECT
private:
    QPalette palette;
public:
    tblFgClrPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged();
};

class tblRearrangePropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    bool value;
public:
    tblRearrangePropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};

class tblRowStripingPropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    bool value;
public:
    tblRowStripingPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};

class tblEnablePropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    bool value;
public:
    tblEnablePropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};

class tblSelHighlightPropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    bool value;
public:
    tblSelHighlightPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};



typedef enum {tblPPAll, tblPPNone, tblPPVisible} tblPickeableParts;
class tblPickablePartsPropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    tblPickeableParts value;
public:
    tblPickablePartsPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};

class tblHittestPropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    bool value;
public:
    tblHittestPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
  public slots:
    virtual void valueChooserChanged(QString text);
};

class tblDataConfig{
private:
  int rowCount;
  int colCount;
  QStringList colFmt;
  QStringList colWidth;
  QStringList rowTitle;
  QStringList colTitle;
  QStringList colEdit;
  bool useNumberedRow;
  bool useNumberedCol;
  char *** data;
  QString baseDef;
public:
    tblDataConfig(QString cfg);
    int getRowCount(void);
    int getColCount(void);
    QString getColFmt(int);
    QString getColWidth(int);
    QString getRowTitle(int);
    QString getColTitle(int);
    QString getColEdit(int);
    QString getData(int f, int c);
    QString getBaseDef(void);
    bool getNumberedRows(void);
    bool getNumberedCols(void);
    ~tblDataConfig(void);
};

class updButton : public QPushButton, public updPropWdg{
public:
    updButton(abstractPropEditor * prop);
     virtual void updPropValue();
};

class tblDataPropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    QString value;
    QString genColWidth(tblDataConfig &);
    QString genColFomat(tblDataConfig &);
    QString genColNames(tblDataConfig &);
    QString genColEditable(tblDataConfig &);
    QString genRowNames(tblDataConfig &);
    QString genData(tblDataConfig &);
public:
    tblDataPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QWidget * getEditWidget();
    virtual QString generateCode();
  public slots:
    virtual void clicked(bool);
};

class tblFontNamePropEditor: public abstractPropEditor
{
    Q_OBJECT
private:
    QFontDatabase fontDB;
public:
    tblFontNamePropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void fontChanged(const QString &txt);
};

class tblFontSizePropEditor: public abstractPropEditor
{
    Q_OBJECT
public:
    tblFontSizePropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void fontSizeChanged(const QString & text);
};

class tblFontWeightPropEditor: public abstractPropEditor
{
    Q_OBJECT
public:
    tblFontWeightPropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void fontWeightChanged(const QString & text);
};

class tblFontAnglePropEditor: public abstractPropEditor
{
    Q_OBJECT
public:
    tblFontAnglePropEditor(abstractUICCtrl *ctrl);
    virtual QString getValue();
    virtual bool isValidValue(QString newVal);
    virtual void setValue(QString newVal);
    virtual QString generateCode();
    virtual QWidget * getEditWidget();
  public slots:
    virtual void fontAngleChanged(const QString & text);
};

class tableCtrl : public abstractUICCtrl
{
private:
  static unsigned int uicNameCounter;

public:
    static unsigned int getNameCounter();
    tableCtrl(QWidget * parent, abstractUICCtrl *octaveParent, childWndDlg * parentWnd);
    void setParents(QWidget * parent, abstractUICCtrl * octaveParent, childWndDlg * parentWnd);
    QStringList generateMFile(QString path);
    virtual QString className() { return "tableCtrl";}
    abstractUICCtrl * clone(QWidget *parent,
                            abstractUICCtrl *octaveParent,
                            childWndDlg * parentWnd);
    void setHidden(bool);
    virtual ~tableCtrl();
};

class tableCtrlGen: public controlGenerator
{
public:
  tableCtrlGen();
  virtual abstractUICCtrl * getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg);
};

class tableCtrlGenV1: public controlGenerator
{
public:
  tableCtrlGenV1();
  virtual abstractUICCtrl * getControl(QXmlStreamReader &xml, abstractUICCtrl *parent, childWndDlg * dlg);
};



#endif // TABLECTRL_H


